#
import pytest
from load_one_data import dump_diff, dump_same_and_not_same, one_dump_test

from ArWikiCats import resolve_label_ar

data0_no_label = {
    "Category:1st-millennium architecture": "تصنيف:عمارة الألفية 1",
    "Category:10th-century BC architecture": "تصنيف:عمارة القرن 10 ق م",
    "Category:1st-century architecture": "تصنيف:عمارة القرن 1",
    "Category:1370s conflicts": "تصنيف:نزاعات في عقد 1370",
    "Attacks on Jewish institutions by century": "تصنيف:هجمات على مؤسسات يهودية حسب القرن",
    "Attacks on Jewish institutions in Poland": "تصنيف:هجمات على مؤسسات يهودية في بولندا",
    "Attacks on Jewish institutions in United States": "تصنيف:هجمات على مؤسسات يهودية في الولايات المتحدة",
    "Attacks on Jewish institutions": "تصنيف:هجمات على مؤسسات يهودية",
    "18th-century attacks on Jewish institutions": "تصنيف:هجمات القرن 18 في مؤسسات يهودية",
    "20th-century attacks on Jewish institutions in United States": "تصنيف:هجمات القرن 20 في مؤسسات يهودية في الولايات المتحدة",
    "20th-century attacks on Jewish institutions": "تصنيف:هجمات القرن 20 في مؤسسات يهودية",
    "21st-century attacks on Jewish institutions in United States": "تصنيف:هجمات القرن 21 في مؤسسات يهودية في الولايات المتحدة",
    "21st-century attacks on Jewish institutions": "تصنيف:هجمات القرن 21 في مؤسسات يهودية",
    "Lists of people executed in the United States by year": "قوائم أشخاص أعدموا في الولايات المتحدة حسب السنة",
    "Lists of people executed in United States by year": "قوائم أشخاص أعدموا في الولايات المتحدة حسب السنة",
    "Healthcare by city of the United States": "الرعاية الصحية بواسطة مدينة الولايات المتحدة",
    "north american television awards": "جوائز التلفزة الأمريكية الشمالية",
    "mexican revolution films": "أفلام الثورة المكسيكية",
    "northern-ireland football cups": "كؤوس كرة القدم الأيرلندية الشمالية",
    "chinese professional baseball league awards": "جوائز دوري كرة القاعدة الصيني للمحترفين",
    "paralympics people": "أشخاص في الألعاب البارالمبية",
    "Fictional people executed for murder": "أشخاص خياليون أعدموا بتهمة قتل",
    "Fictional people executed for treason": "أشخاص خياليون أعدموا بتهمة خيانة",
    "Buddhist comics": "قصص مصورة بوذيون",
    "Buddhist media in Taiwan": "إعلام بوذيون في تايوان",
    "Buddhist media": "إعلام بوذيون",
    "Buddhist music": "موسيقى بوذيون",
    "Mexican television awards": "جوائز التلفزة المكسيكية",
    "2025–26 in Northern Ireland association football": "كرة القدم الأيرلندية الشمالية في 2025–26",
    "1971–72 in Northern Ireland association football": "كرة القدم الأيرلندية الشمالية في 1971–72",
    "Buddhist video games": "ألعاب فيديو بوذيون",
    "Hindu music": "موسيقى هندوس",
    "Islamic media in India": "إعلام إسلاميون في الهند",
    "Islamic media": "إعلام إسلاميون",
    "Islamic music": "موسيقى إسلاميون",
    "Nazi culture": "ثقافة نازيون",
    "Nazi songs": "أغاني نازيون",
    "Saints and Soldiers films": "قديسون وأفلام مجندون",
    "muslim people templates": "قوالب أعلام مسلمون",
    "deaf culture": "ثقافة صم",
    "singaporean blind people": "أعلام سنغافوريون مكفوفون",
    "ukrainian deaf people": "أعلام أوكرانيون صم",
    "slovenian deaf people": "أعلام سلوفينيون صم",
    "russian blind people": "أعلام روس مكفوفون",
    "czech deaf people": "أعلام تشيكيون صم",
    "by benjamin britten": "بواسطة بنجامين بريتن",
    "by james cameron": "بواسطة جيمس كاميرون",
    "by raphael": "بواسطة رافاييل",
    "by vaikom muhammad basheer": "بواسطة محمد بشير",
    "expatriate men's footballers": "لاعبو كرة قدم مغتربون",
    "expatriate men's soccer players": "لاعبو كرة قدم مغتربون",
    "Icelandic deaf people": "أعلام آيسلنديون صم",
    "Romantic composers": "ملحنون رومانسيون",
    "Expatriate men's footballers in Papua New Guinea": "لاعبو كرة قدم مغتربون في بابوا غينيا الجديدة",
    "American expatriate men's soccer players": "لاعبو كرة قدم أمريكيون مغتربون",
    "Byzantine female saints": "قديسات بيزنطيات",
    "Ancient Christians": "مسيحيون قدماء",
    "Ancient Christian female saints": "قديسات مسيحيات قدماء",
    "Ancient Jewish physicians": "أطباء يهود قدماء",
    "Ancient Jewish scholars": "دارسون يهود قدماء",
    "Ancient Jewish women": "يهوديات قدماء",
    "Ancient Jewish writers": "كتاب يهود قدماء",
    "Murdered American Jews": "أمريكيون يهود قتلوا",
}

data0 = {
    "Category:1630s natural disasters": "",
    "Category:17th-century plays": "",
    "Category:1830s documents": "",
    "Category:1990s television series debuts": "",
    "Category:19th-century books": "",
    "Category:3rd-century paintings": "",
    "Category:3rd-millennium works": "",
    "Category:2026 television series debuts": "",
    "Category:13th-century plays": "",
    "Category:1009 works": "تصنيف:أعمال 1009",
    "Category:1239 works": "تصنيف:أعمال 1239",
    "Category:1261 works": "تصنيف:أعمال 1261",
    "Category:1429 in law": "تصنيف:قانون في 1429",
    "Category:1483 works": "تصنيف:أعمال 1483",
    "Category:1562 works": "تصنيف:أعمال 1562",
    "Category:1701 in international relations": "تصنيف:العلاقات الدولية في 1701",
    "Category:1727 in law": "تصنيف:قانون في 1727",
    "Category:1737 in music": "تصنيف:الموسيقى في 1737",
    "Category:1752 works": "تصنيف:أعمال 1752",
    "Category:1793 elections": "تصنيف:انتخابات 1793",
    "Category:1795 in mass media": "تصنيف:وسائل إعلام في 1795",
    "Category:1826 murders by continent": "تصنيف:جرائم قتل في 1826 حسب القارة",
    "Category:1832 works": "تصنيف:أعمال 1832",
    "Category:1833 in mass media": "تصنيف:وسائل إعلام في 1833",
    "Category:1835 meteorology": "تصنيف:الأرصاد الجوية 1835",
    "Category:1842 in mass media": "تصنيف:وسائل إعلام في 1842",
    "Category:1854 in law": "تصنيف:قانون في 1854",
    "Category:1855 disasters by country": "تصنيف:كوارث 1855 حسب البلد",
    "Category:1867 in mass media": "تصنيف:وسائل إعلام في 1867",
    "Category:1902 in music by country": "تصنيف:موسيقى في 1902 حسب البلد",
    "Category:1904 murders by country": "تصنيف:جرائم قتل في 1904 حسب البلد",
    "Category:1910 in international relations": "تصنيف:العلاقات الدولية في 1910",
    "Category:1917 in music": "تصنيف:الموسيقى في 1917",
    "Category:1934 murders": "تصنيف:جرائم قتل في 1934",
    "Category:1948 in law": "تصنيف:قانون في 1948",
    "Category:1948 treaties": "تصنيف:اتفاقيات 1948",
    "Category:1965 in music by country": "تصنيف:موسيقى في 1965 حسب البلد",
    "Category:1970s elections": "تصنيف:انتخابات عقد 1970",
    "Category:1972 murders": "تصنيف:جرائم قتل في 1972",
    "Category:1990 in international relations": "تصنيف:العلاقات الدولية في 1990",
    "Category:2021 meteorology": "تصنيف:الأرصاد الجوية 2021",
    "Category:2024 disasters by country": "تصنيف:كوارث 2024 حسب البلد",
    "Category:60s conflicts": "تصنيف:نزاعات عقد 60",
    "Category:April 1959 in sports": "تصنيف:أبريل 1959 في ألعاب رياضية",
    "Category:December 2007 in sports": "تصنيف:ديسمبر 2007 في ألعاب رياضية",
    "Category:Fiction set in 1716": "تصنيف:الخيال تقع أحداثها في 1716",
    "Category:Fiction set in 1790s": "تصنيف:الخيال تقع أحداثها في عقد 1790",
    "Category:Fiction set in 1819": "تصنيف:الخيال تقع أحداثها في 1819",
    "Category:Fiction set in 27th century": "تصنيف:الخيال تقع أحداثها في القرن 27",
    "Category:January 1901 in sports": "تصنيف:يناير 1901 في ألعاب رياضية",
    "Category:January 1952 in sports": "تصنيف:يناير 1952 في ألعاب رياضية",
    "Category:May 1946 in sports": "تصنيف:مايو 1946 في ألعاب رياضية",
    "Category:May 1993 in sports": "تصنيف:مايو 1993 في ألعاب رياضية",
    "Category:September 2024 in sports": "تصنيف:سبتمبر 2024 في ألعاب رياضية",
    "Category:Transport companies established in 1946": "تصنيف:شركات النقل أسست في 1946",
    "Category:Transport infrastructure completed in 1916": "تصنيف:البنية التحتية للنقل اكتملت في 1916",
    "Category:Transport infrastructure completed in 1972": "تصنيف:البنية التحتية للنقل اكتملت في 1972",
    "Category:Transport infrastructure completed in 21st century": "تصنيف:البنية التحتية للنقل اكتملت في القرن 21",
    "Category:Articles containing potentially dated statements from February 2024": "",
    "Category:Articles lacking reliable references from February 2013": "",
    "Category:Articles with dead external links from June 2017": "",
    "Category:Educational institutions established in 2009": "",
    "Category:1013 works": "تصنيف:أعمال 1013",
    "Category:1817 treaties": "تصنيف:اتفاقيات 1817",
    "Category:1999 disasters by country": "تصنيف:كوارث 1999 حسب البلد",
    "Category:1909 murders by country": "تصنيف:جرائم قتل في 1909 حسب البلد",
    "Category:1265 works": "تصنيف:أعمال 1265",
    "Category:October 1922 in sports": "تصنيف:أكتوبر 1922 في ألعاب رياضية",
    "Category:1708 in law": "تصنيف:قانون في 1708",
    "Category:June 1956 in sports": "تصنيف:يونيو 1956 في ألعاب رياضية",
    "Category:Articles containing potentially dated statements from April 2014": "",
    "Category:January 2026 in sports": "تصنيف:يناير 2026 في ألعاب رياضية",
    "Category:July 2026 in sports": "تصنيف:يوليو 2026 في ألعاب رياضية",
    "Category:2026 elections by country": "تصنيف:انتخابات 2026 حسب البلد",
    "Category:2026 in music by country": "تصنيف:موسيقى في 2026 حسب البلد",
    "Category:April 2026 in sports": "تصنيف:أبريل 2026 في ألعاب رياضية",
    "Category:August 2026 in sports": "تصنيف:أغسطس 2026 في ألعاب رياضية",
    "Category:February 2026 in sports": "تصنيف:فبراير 2026 في ألعاب رياضية",
    "Category:June 2026 in sports": "تصنيف:يونيو 2026 في ألعاب رياضية",
    "Category:March 2026 in sports": "تصنيف:مارس 2026 في ألعاب رياضية",
    "Category:May 2026 in sports": "تصنيف:مايو 2026 في ألعاب رياضية",
    "Category:November 2026 in sports": "تصنيف:نوفمبر 2026 في ألعاب رياضية",
    "Category:September 2026 in sports": "تصنيف:سبتمبر 2026 في ألعاب رياضية",
    "20th-century Anglican archbishops in Ireland": "رؤساء أساقفة أنجليكيون في أيرلندا في القرن 20",
    "20th-century Anglican archbishops in New Zealand": "رؤساء أساقفة أنجليكيون في نيوزيلندا في القرن 20",
    "21st-century Anglican archbishops in New Zealand": "رؤساء أساقفة أنجليكيون في نيوزيلندا في القرن 21",
    "17th-century Dutch books": "كتب هولندية في القرن 17",
}

data1 = {}

to_test = [
    # ("test_2_skip2_0", data0),
    ("test_2_skip2_2", data1),
]


@pytest.mark.parametrize("category, expected", data0.items(), ids=data0.keys())
@pytest.mark.skip2
def test_2_skip2_0(category: str, expected: str) -> None:
    label = resolve_label_ar(category)
    assert label == expected


@pytest.mark.parametrize("category, expected", data0_no_label.items(), ids=data0_no_label.keys())
@pytest.mark.skip2
def test_2_skip2_2(category: str, expected: str) -> None:
    label = resolve_label_ar(category)
    assert label == expected


@pytest.mark.parametrize("name,data", to_test)
@pytest.mark.dump
def test_peoples(name: str, data: dict[str, str]) -> None:
    expected, diff_result = one_dump_test(data, resolve_label_ar)

    dump_diff(diff_result, name)
    dump_same_and_not_same(data, expected, name)
    assert diff_result == expected, f"Differences found: {len(diff_result):,}, len all :{len(data):,}"



import pytest

from ArWikiCats.format_bots.relation_mapping import translation_category_relations
from ArWikiCats.utils.match_relation_word import get_relation_word, get_relation_word_new

# ===============================================
# Test: Single relation word detection
# ===============================================

data_test2 = [
    ("Schools for-deaf in New York (state)", "for-deaf", "للصم"),
    ("Cabinets involving the Liberal Party (Norway)", "involving the", "تشمل"),
    ("Television plays directed by William Sterling (director)", "directed by", "أخرجها"),
    ("Television plays filmed in Brisbane", "filmed in", "صورت في"),
    ("Television personalities from Yorkshire", "from", "من"),
    # --- Additional logical examples ---
    ("Works published by Oxford University Press", "published by", "نشرتها"),
    ("Bridges built in 1885", "built in", "بنيت في"),
    ("Ships launched in 1910", "launched in", "أطلقت في"),
    ("Films written by John Doe", "written by", "كتبها"),
    ("Cities established in 1750", "established in", "أسست في"),
]


@pytest.mark.parametrize(
    "category, expected_key, expected_value",
    data_test2,
    ids=[x[0] for x in data_test2],
)
def test_single_relation(category: str, expected_key: str, expected_value: str) -> None:
    key, value = get_relation_word(category, translation_category_relations)
    assert key.strip() == expected_key
    assert value == expected_value


@pytest.mark.parametrize(
    "category, expected_key, expected_value",
    data_test2,
    ids=[x[0] for x in data_test2],
)
def test_single_relation_compare(category: str, expected_key: str, expected_value: str) -> None:
    key1, value1 = get_relation_word(category, translation_category_relations)
    key2, value2 = get_relation_word_new(category, translation_category_relations)
    assert key2 == key1
    assert value2 == value1


# ===============================================
# Test: Multiple relation words in the same line
# ===============================================


@pytest.mark.parametrize(
    "category, first_expected_key, first_expected_value",
    [
        # --- User examples (2 relation words) ---
        ("100 metres at the African Championships by Athletics", "by", "حسب"),
        ("100 metres at the IAAF World Youth Championships by Athletics", "by", "حسب"),
        ("100 metres at the World Para Athletics Championships", "at", ""),
        ("Documentary films about the 2011 Tōhoku earthquake and tsunami", "about", "عن"),
        ("People charged with lèse majesté in Thailand", "charged with", "أتهموا بتهمة"),
        ("People associated with former colleges of the University of London", "associated with", "مرتبطة مع"),
        ("People associated with Nazarene universities and colleges", "associated with", "مرتبطة مع"),
        # --- Extra cases with two relation keys ---
        ("Songs written by John Smith and produced in London", "written by", "كتبها"),
    ],
)
def test_multiple_relations_first_match(category: str, first_expected_key: str, first_expected_value: str) -> None:
    """Ensure that only the first matching relation word is returned."""
    key, value = get_relation_word(category, translation_category_relations)
    assert key.strip() == first_expected_key
    assert value == first_expected_value


def test_multiple_relations() -> None:
    """Ensure that only the first matching relation word is returned."""
    category = "Ships built in Germany and France launched in 1900"
    key, value = get_relation_word(category, translation_category_relations)
    assert key.strip() in ["built in", "launched in"]
    assert value in ["بنيت في", "أطلقت في"]


# ===============================================
# Test: No relation found
# ===============================================


@pytest.mark.parametrize(
    "category",
    [
        "Random topic with no relation word here",
        "Mountain ranges of-the world",
        "Unclassified biological samples",
    ],
)
def test_no_relation(category: str) -> None:
    key, value = get_relation_word(category, translation_category_relations)
    assert key == ""
    assert value == ""


# ===============================================
# Test: Relation word appears but without surrounding spaces
# Should NOT match (because the function requires ' key ' with spaces)
# ===============================================


@pytest.mark.parametrize(
    "category, wrong_rel",
    [
        ("Schools forthedeaf USA", "for-deaf"),
        ("Bridges builtin London", "built in"),
        ("Items producedby students", "produced by"),
    ],
)
def test_relation_not_matched_without_spaces(category: str, wrong_rel: str) -> None:
    key, value = get_relation_word(category, translation_category_relations)
    assert key == ""
    assert value == ""


# ===============================================
# Test: Ensure the search is sensitive to ordering of dict
# The first key in the dict that appears should win.
# ===============================================


def test_first_match_priority() -> None:
    """If both 'in' and 'involving' appear, 'involving' should match first only if it appears first in mapping."""
    cat = "Topic involving science in Europe"

    # mapping order: "involving" comes before "in"
    key, value = get_relation_word(cat, translation_category_relations)
    assert key.strip() == "involving"
    assert value == "تشمل"



#!/usr/bin/python3
"""Integration tests for v3i translations resolvers validating country, year, and combined formatters."""

import pytest

from ArWikiCats.new_resolvers.translations_resolvers_v3i.resolve_v3ii import resolve_year_job_countries

test_data = {
    "18th-century princes": "أمراء في القرن 18",
    "18th-century nobility": "نبلاء في القرن 18",
    "21st-century yemeni writers": "كتاب يمنيون في القرن 21",
    "21st-century New Zealand writers": "كتاب نيوزيلنديون في القرن 21",
    # "20th century american people": "أمريكيون في القرن 20",
    "20th century american people": "أمريكيون في القرن 20",
}


@pytest.mark.parametrize("category,expected", test_data.items(), ids=test_data.keys())
def test_resolve_v3ii(category: str, expected: str) -> None:
    """
    pytest tests/translations_resolvers_v3i/test_resolve_v3ii.py::test_data
    """
    result = resolve_year_job_countries(category)
    assert result == expected



#!/usr/bin/python3
"""Integration tests for v3i translations resolvers validating country, year, and combined formatters."""

import pytest

from ArWikiCats.new_resolvers.translations_resolvers_v3i.resolve_v3i import (
    multi_bot_v4,
    resolve_year_job_from_countries,
)

bot = multi_bot_v4()


class TestCountriesPart:
    test_data_standard = {
        "writers from Crown of Aragon": "كتاب من تاج أرغون",
        "writers from yemen": "كتاب من اليمن",
    }

    @pytest.mark.parametrize("category,expected", test_data_standard.items(), ids=test_data_standard.keys())
    def test_country_combinations(self, category: str, expected: str) -> None:
        """
        Test
        """
        result = bot.country_bot.search(category)
        assert result == expected


class TestYearPart:
    test_data_standard = {
        "100s": "عقد 100",
    }

    @pytest.mark.parametrize("category,expected", test_data_standard.items(), ids=test_data_standard.keys())
    def test_year_combinations(self, category: str, expected: str) -> None:
        """
        Test
        """
        result = bot.other_bot.search_all(category)
        assert result == expected


class TestAllParts:
    test_data_standard = {
        "20th-century non-fiction writers from russian empire": "كتاب غير روائيين من الإمبراطورية الروسية في القرن 20",
        "14th-century writers from Crown of Aragon": "كتاب من تاج أرغون في القرن 14",
        "14th-century writers from yemen": "كتاب من اليمن في القرن 14",
        "18th-century non-fiction writers from Russian Empire": "كتاب غير روائيين من الإمبراطورية الروسية في القرن 18",
        "18th-century non-fiction writers from the Russian Empire": "كتاب غير روائيين من الإمبراطورية الروسية في القرن 18",
        "18th-century sculptors from Bohemia": "نحاتون من بوهيميا في القرن 18",
        "19th-century non-fiction writers from Russian Empire": "كتاب غير روائيين من الإمبراطورية الروسية في القرن 19",
        "20th-century architects from Northern Ireland": "معماريون من أيرلندا الشمالية في القرن 20",
        "20th-century engineers from Northern Ireland": "مهندسون من أيرلندا الشمالية في القرن 20",
        "20th-century non-fiction writers from Northern Ireland": "كتاب غير روائيين من أيرلندا الشمالية في القرن 20",
        "20th-century non-fiction writers from northern ireland": "كتاب غير روائيين من أيرلندا الشمالية في القرن 20",
        "21st-century engineers from Northern Ireland": "مهندسون من أيرلندا الشمالية في القرن 21",
        "21st-century male actors from Georgia (country)": "ممثلون ذكور من جورجيا في القرن 21",
        "21st-century male artists from Northern Ireland": "فنانون ذكور من أيرلندا الشمالية في القرن 21",
        "21st-century medical doctors from Northern Ireland": "أطباء من أيرلندا الشمالية في القرن 21",
        "21st-century non-fiction writers from Northern Ireland": "كتاب غير روائيين من أيرلندا الشمالية في القرن 21",
        "21st-century painters from Northern Ireland": "رسامون من أيرلندا الشمالية في القرن 21",
        "9th-century historians from al-Andalus": "مؤرخون من الأندلس في القرن 9",
        "17th-century historians from Bohemia": "مؤرخون من بوهيميا في القرن 17",
        "19th-century biographers from Russian Empire": "كتاب سيرة من الإمبراطورية الروسية في القرن 19",
        "14th-century people from Holy Roman Empire": "أشخاص من الإمبراطورية الرومانية المقدسة في القرن 14",
    }

    @pytest.mark.parametrize("category,expected", test_data_standard.items(), ids=test_data_standard.keys())
    def test_year_country_combinations(self, category: str, expected: str) -> None:
        """
        Test
        """
        result = resolve_year_job_from_countries(category)
        assert result == expected

    data_2 = {
        # "21st-century musicians by instrument from Northern Ireland": "موسيقيون في القرن 21 حسب الآلة من أيرلندا الشمالية",
        # "20th-century musicians by instrument from Northern Ireland": "موسيقيون في القرن 20 حسب الآلة من أيرلندا الشمالية",
        "9th-century people from East Francia": "أشخاص من مملكة الفرنجة الشرقيين في القرن 9",
        "9th-century people from West Francia": "أشخاص من مملكة الفرنجة الغربيين في القرن 9",
        "21st-century women educators from Northern Ireland": "معلمات من أيرلندا الشمالية في القرن 21",
        "21st-century women medical doctors from Northern Ireland": "طبيبات من أيرلندا الشمالية في القرن 21",
        "20th century people from al-andalus": "أشخاص من الأندلس في القرن 20",
        "18th-century women singers from the Holy Roman Empire": "مغنيات من الإمبراطورية الرومانية المقدسة في القرن 18",
        "10th-century people from West Francia": "أشخاص من مملكة الفرنجة الغربيين في القرن 10",
        "17th-century women from the Republic of Venice": "نساء من جمهورية البندقية في القرن 17",
    }

    @pytest.mark.parametrize("category,expected", data_2.items(), ids=data_2.keys())
    def test_data_2(self, category: str, expected: str) -> None:
        """
        Test
        """
        result = resolve_year_job_from_countries(category)
        assert result == expected

    deaths_data = {
        "15th-century deaths from cancer": "وفيات بسبب السرطان في القرن 15",
    }

    @pytest.mark.parametrize("category,expected", deaths_data.items(), ids=deaths_data.keys())
    def test_deaths_data(self, category: str, expected: str) -> None:
        """
        pytest tests/translations_resolvers_v3i/test_resolve_v3i.py::TestAllParts::test_deaths_data
        """
        result = resolve_year_job_from_countries(category)
        assert result == expected



#
import pytest

from ArWikiCats import resolve_label_ar

data0_no_label = {
    "Defunct Christian organizations based in United States": "تصنيف:منظمات مسيحية سابقة مقرها في الولايات المتحدة",
    "Defunct Christian schools in Canada": "تصنيف:مدارس مسيحية سابقة في كندا",
    "Defunct Christian schools in United States": "تصنيف:مدارس مسيحية سابقة في الولايات المتحدة",
    "Defunct Islamic organizations based in United States": "تصنيف:منظمات إسلامية سابقة مقرها في الولايات المتحدة",
    "Defunct Jewish organizations based in United States": "تصنيف:منظمات يهودية سابقة مقرها في الولايات المتحدة",
    "Defunct national sports teams by country": "فرق رياضية وطنية سابقة حسب البلد",
    "defunct american football venues": "ملاعب كرة قدم أمريكية سابقة",
    "defunct amusement parks": "متنزهات ملاهي سابقة",
    "defunct art museums and galleries": "متاحف فنية ومعارض سابقة",
    "defunct asian restaurants": "مطاعم آسيوية سابقة",
    "defunct athletics venues": "ملاعب ألعاب قوى سابقة",
    "defunct baseball venues": "ملاعب كرة قاعدة سابقة",
    "defunct basketball venues": "ملاعب كرة سلة سابقة",
    "defunct clubs and societies": "أندية وجمعيات سابقة",
    "defunct comedy clubs": "أندية كوميدية سابقة",
    "defunct communist parties": "أحزاب شيوعية سابقة",
    "defunct cycling races": "سباقات سباق دراجات هوائية سابقة",
    "defunct elementary schools": "مدارس إبتدائية سابقة",
    "defunct european restaurants": "مطاعم أوروبية سابقة",
    "defunct far-right political parties": "أحزاب اليمين المتطرف سابقة",
    "defunct feminist organizations": "منظمات نسوية سابقة",
    "defunct football clubs": "أندية كرة قدم سابقة",
    "defunct football venues": "ملاعب كرة قدم سابقة",
    "defunct french restaurants": "مطاعم فرنسية سابقة",
    "defunct golf tournaments": "بطولات غولف سابقة",
    "defunct high schools": "مدارس ثانوية سابقة",
    "defunct ice hockey venues": "ملاعب هوكي جليد سابقة",
    "defunct italian restaurants": "مطاعم إيطالية سابقة",
    "defunct japanese restaurants": "مطاعم يابانية سابقة",
    "defunct liberal parties": "أحزاب ليبرالية سابقة",
    "defunct lower houses": "المجالس الدنيا سابقة",
    "defunct mexican restaurants": "مطاعم مكسيكية سابقة",
    "defunct middle schools": "مدارس إعدادية سابقة",
    "defunct motorsport venues": "ملاعب رياضة محركات سابقة",
    "defunct multi-national basketball leagues": "دوريات كرة سلة متعددة الجنسيات سابقة",
    "defunct private schools": "مدارس خاصة سابقة",
    "defunct private universities and colleges": "جامعات وكليات خاصة سابقة",
    "defunct professional sports leagues": "دوريات رياضية للمحترفين سابقة",
    "defunct public high schools": "مدارس ثانوية عامة سابقة",
    "defunct radio networks": "شبكات مذياع سابقة",
    "defunct radio stations": "محطات إذاعية سابقة",
    "defunct railway stations": "محطات السكك الحديدية سابقة",
    "defunct right-wing parties": "أحزاب يمينية سابقة",
    "defunct rugby league venues": "ملاعب دوري رجبي سابقة",
    "defunct rugby union stadiums": "ملاعب اتحاد رجبي سابقة",
    "defunct soccer clubs": "أندية كرة قدم سابقة",
    "defunct soccer venues": "ملاعب كرة قدم سابقة",
    "defunct socialist parties": "أحزاب اشتراكية سابقة",
    "defunct softball venues": "ملاعب كرة لينة سابقة",
    "defunct sports governing bodies": "هيئات تنظيم رياضية سابقة",
    "defunct television channels": "قنوات تلفزيونية سابقة",
    "defunct television networks": "شبكات تلفزيونية سابقة",
    "defunct tennis tournaments": "بطولات كرة مضرب سابقة",
    "defunct tourist attractions": "مواقع جذب سياحي سابقة",
    "defunct trade unions": "نقابات عمالية سابقة",
    "defunct women's basketball competitions": "منافسات كرة سلة نسائية سابقة",
    "defunct women's basketball leagues": "دوريات كرة سلة نسائية سابقة",
    "defunct women's football clubs": "أندية كرة قدم نسائية سابقة",
    "defunct women's soccer leagues": "دوريات كرة قدم نسائية سابقة",
}


@pytest.mark.parametrize("category, expected", data0_no_label.items(), ids=data0_no_label.keys())
@pytest.mark.skip2
def test_2_skip2_2(category: str, expected: str) -> None:
    label = resolve_label_ar(category)
    assert label == expected



# -*- coding: utf-8 -*-
from .config import all_params as config_all_params
from .event_processing import (
    EventProcessor,
    batch_resolve_labels,
    resolve_arabic_category_label,
)
from .helps.len_print import dump_all_len
from .helps.log import LoggerWrap, logger
from .helps.memory import print_memory
from .main_processers.main_resolve import resolve_label_ar

__version__ = "0.1.0b3"

__all__ = [
    "logger",
    "LoggerWrap",
    "resolve_label_ar",
    "batch_resolve_labels",
    "resolve_arabic_category_label",
    "EventProcessor",
    "do_print_options",
    "print_memory",
    "dump_all_len",
    "config_all_params",
]



#!/usr/bin/python3
"""Integration tests for format_multi_data  """

import pytest

from ArWikiCats.translations_formats import FormatDataV2, MultiDataFormatterBaseV2


@pytest.fixture
def multi_bot() -> MultiDataFormatterBaseV2:
    nationality_data = {
        "United States": {"ar": "الولايات المتحدة"},
        "yemen": {"ar": "اليمن"},
    }

    formatted_data = {
        "{en}": "{ar}",
        "Olympic gold medalists for {en}": "فائزون بميداليات ذهبية أولمبية من {ar}",
        "Olympic gold medalists for {en} in alpine skiing": "فائزون بميداليات ذهبية أولمبية من {ar} في التزلج على المنحدرات الثلجية",
        "Olympic gold medalists for {en} in {en_sport}": "فائزون بميداليات ذهبية أولمبية من {ar} في {sport_ar}",
    }

    sport_data = {
        "alpine skiing": {
            "sport_ar": "التزلج على المنحدرات الثلجية",
        },
        "football": {
            "sport_ar": "كرة القدم",
        },
    }

    country_bot = FormatDataV2(
        formatted_data=formatted_data,
        data_list=nationality_data,
        key_placeholder="{en}",
    )

    other_bot = FormatDataV2(
        {},
        sport_data,
        key_placeholder="{en_sport}",
    )

    return MultiDataFormatterBaseV2(
        country_bot=country_bot,
        other_bot=other_bot,
        search_first_part=True,
    )


test_data_1 = {
    "United States": "الولايات المتحدة",
    "Olympic gold medalists for United States": "فائزون بميداليات ذهبية أولمبية من الولايات المتحدة",
    "Olympic gold medalists for United States in alpine skiing": "فائزون بميداليات ذهبية أولمبية من الولايات المتحدة في التزلج على المنحدرات الثلجية",
    # "Olympic gold medalists for the United States in football": "فائزون بميداليات ذهبية أولمبية من الولايات المتحدة في كرة القدم",
}


@pytest.mark.parametrize("category, expected", test_data_1.items(), ids=test_data_1.keys())
@pytest.mark.fast
def test_multi_bot(multi_bot: MultiDataFormatterBaseV2, category: str, expected: str) -> None:
    result = multi_bot.search_all(category)
    assert result == expected



"""


"""

import pytest
from load_one_data import dump_diff, one_dump_test

from ArWikiCats import resolve_arabic_category_label
from ArWikiCats.legacy_bots.matables_bots.data import Keep_it_last


@pytest.mark.fast
def test_Keep_it_last() -> None:
    # Arabic translation of term "remakes of" ("معاد إنتاجها") should be at last:

    result = resolve_arabic_category_label("Category:remakes of historical documents")
    assert result == "تصنيف:وثائق تاريخية معاد إنتاجها"


fast_data = {
    "Category:Remakes of American films": "تصنيف:أفلام أمريكية معاد إنتاجها",
    "Category:Remakes of Argentine films": "تصنيف:أفلام أرجنتينية معاد إنتاجها",
    "Category:Remakes of Australian films": "تصنيف:أفلام أسترالية معاد إنتاجها",
    "Category:Remakes of Austrian films": "تصنيف:أفلام نمساوية معاد إنتاجها",
    "Category:Remakes of Belgian films": "تصنيف:أفلام بلجيكية معاد إنتاجها",
    "Category:Remakes of Brazilian films": "تصنيف:أفلام برازيلية معاد إنتاجها",
    "Category:Remakes of British films": "تصنيف:أفلام بريطانية معاد إنتاجها",
    "Category:Remakes of Burmese films": "تصنيف:أفلام بورمية معاد إنتاجها",
    "Category:Remakes of Canadian films": "تصنيف:أفلام كندية معاد إنتاجها",
    "Category:Remakes of Chilean films": "تصنيف:أفلام تشيلية معاد إنتاجها",
    "Category:Remakes of Chinese films": "تصنيف:أفلام صينية معاد إنتاجها",
    "Category:Remakes of Danish films": "تصنيف:أفلام دنماركية معاد إنتاجها",
    "Category:Remakes of Dutch films": "تصنيف:أفلام هولندية معاد إنتاجها",
    "Category:Remakes of Finnish films": "تصنيف:أفلام فنلندية معاد إنتاجها",
    "Category:Remakes of French films": "تصنيف:أفلام فرنسية معاد إنتاجها",
    "Category:Remakes of German films": "تصنيف:أفلام ألمانية معاد إنتاجها",
    "Category:Remakes of Hong Kong films": "تصنيف:أفلام هونغ كونغية معاد إنتاجها",
    "Category:Remakes of Hungarian films": "تصنيف:أفلام مجرية معاد إنتاجها",
    "Category:Remakes of Icelandic films": "تصنيف:أفلام آيسلندية معاد إنتاجها",
    "Category:Remakes of Indian films": "تصنيف:أفلام هندية معاد إنتاجها",
    "Category:Remakes of Indian television series": "تصنيف:مسلسلات تلفزيونية هندية معاد إنتاجها",
    "Category:Remakes of Indonesian films": "تصنيف:أفلام إندونيسية معاد إنتاجها",
    "Category:Remakes of Irish films": "تصنيف:أفلام أيرلندية معاد إنتاجها",
    "Category:Remakes of Italian films": "تصنيف:أفلام إيطالية معاد إنتاجها",
    "Category:Remakes of Japanese films": "تصنيف:أفلام يابانية معاد إنتاجها",
    "Category:Remakes of Malaysian films": "تصنيف:أفلام ماليزية معاد إنتاجها",
    "Category:Remakes of Mexican films": "تصنيف:أفلام مكسيكية معاد إنتاجها",
    "Category:Remakes of Norwegian films": "تصنيف:أفلام نرويجية معاد إنتاجها",
    "Category:Remakes of Pakistani films": "تصنيف:أفلام باكستانية معاد إنتاجها",
    "Category:Remakes of Philippine films": "تصنيف:أفلام فلبينية معاد إنتاجها",
    "Category:Remakes of Russian films": "تصنيف:أفلام روسية معاد إنتاجها",
    "Category:Remakes of South Korean films": "تصنيف:أفلام كورية جنوبية معاد إنتاجها",
    "Category:Remakes of Spanish films": "تصنيف:أفلام إسبانية معاد إنتاجها",
    "Category:Remakes of Sri Lankan films": "تصنيف:أفلام سريلانكية معاد إنتاجها",
    "Category:Remakes of Swedish films": "تصنيف:أفلام سويدية معاد إنتاجها",
    "Category:Remakes of Taiwanese films": "تصنيف:أفلام تايوانية معاد إنتاجها",
    "Category:Remakes of Thai films": "تصنيف:أفلام تايلندية معاد إنتاجها",
    "Category:Remakes of Turkish films": "تصنيف:أفلام تركية معاد إنتاجها",
}


@pytest.mark.parametrize("category, expected", fast_data.items(), ids=fast_data.keys())
def test_Keep_it_last_extended(category: str, expected: str) -> None:
    # Arabic translation of term "remakes of" ("معاد إنتاجها") should be at last:
    label = resolve_arabic_category_label(category)
    assert label == expected


fast_data_empty = {
    "Category:American remakes of Argentine films": "x",
    "Category:American remakes of Belgian films": "x",
    "Category:American remakes of Brazilian films": "x",
    "Category:American remakes of British films": "x",
    "Category:American remakes of Canadian films": "x",
    "Category:American remakes of Danish films": "x",
    "Category:American remakes of Dutch films": "x",
    "Category:American remakes of foreign films": "x",
    "Category:American remakes of French films": "x",
    "Category:American remakes of German films": "x",
    "Category:American remakes of Hong Kong films": "x",
    "Category:American remakes of Indian films": "x",
    "Category:American remakes of Israeli films": "x",
    "Category:American remakes of Italian films": "x",
    "Category:American remakes of Japanese films": "x",
    "Category:American remakes of Mexican films": "x",
    "Category:American remakes of Norwegian films": "x",
    "Category:American remakes of South Korean films": "x",
    "Category:American remakes of Spanish films": "x",
    "Category:American remakes of Swedish films": "x",
    "Category:American remakes of Thai films": "x",
    "Category:Assamese-language remakes of Hindi films": "x",
    "Category:Assamese-language remakes of Malayalam films": "x",
    "Category:Bangladeshi remakes of American films": "x",
    "Category:Bangladeshi remakes of foreign films": "x",
    "Category:Bangladeshi remakes of Indian films": "x",
    "Category:Bangladeshi remakes of Tamil films": "x",
    "Category:British remakes of American films": "x",
    "Category:British remakes of French films": "x",
    "Category:British remakes of German films": "x",
    "Category:Bangladeshi remakes of Telugu films": "x",
    "Category:Bengali remakes of English films": "x",
    "Category:Bengali remakes of Hindi films": "x",
    "Category:Bengali remakes of Kannada films": "x",
    "Category:Bengali remakes of Malayalam films": "x",
    "Category:Bengali remakes of Marathi films": "x",
    "Category:Bengali remakes of Punjabi films": "x",
    "Category:Bengali remakes of Tamil films": "x",
    "Category:Bengali remakes of Telugu films": "x",
    "Category:Bhojpuri remakes of Tamil films": "x",
    "Category:Bhojpuri remakes of Telugu films": "x",
    "Category:Chinese remakes of American films": "x",
    "Category:Chinese remakes of foreign films": "x",
    "Category:Chinese remakes of Japanese films": "x",
    "Category:Chinese remakes of South Korean films": "x",
    "Category:Dutch remakes of British films": "x",
    "Category:Dutch remakes of foreign films": "x",
    "Category:French remakes of American films": "x",
    "Category:Gujarati remakes of Marathi films": "x",
    "Category:Gujarati remakes of Tamil films": "x",
    "Category:Hindi remakes of Bengali films": "x",
    "Category:Hindi remakes of English films": "x",
    "Category:Hindi remakes of Gujarati films": "x",
    "Category:Hindi remakes of Kannada films": "x",
    "Category:Hindi remakes of Malayalam films": "x",
    "Category:Hindi remakes of Marathi films": "x",
    "Category:Hindi remakes of Odia films": "x",
    "Category:Hindi remakes of Punjabi films": "x",
    "Category:Hindi remakes of Rajasthani films": "x",
    "Category:Hindi remakes of silent films": "x",
    "Category:Hindi remakes of Tamil films": "x",
    "Category:Hindi remakes of Telugu films": "x",
    "Category:Hong Kong remakes of American films": "x",
    "Category:Indian remakes of American films": "x",
    "Category:Indian remakes of British films": "x",
    "Category:Indian remakes of foreign films": "x",
    "Category:Indian remakes of French films": "x",
    "Category:Indian remakes of Hong Kong films": "x",
    "Category:Indian remakes of Italian films": "x",
    "Category:Indian remakes of Japanese films": "x",
    "Category:Indian remakes of Pakistani films": "x",
    "Category:Indian remakes of South Korean films": "x",
    "Category:Indian remakes of Spanish films": "x",
    "Category:Indian remakes of Thai films": "x",
    "Category:Iranian remakes of American films": "x",
    "Category:Iranian remakes of foreign films": "x",
    "Category:Italian remakes of foreign films": "x",
    "Category:Italian remakes of French films": "x",
    "Category:Japanese remakes of American films": "x",
    "Category:Japanese remakes of foreign films": "x",
    "Category:Japanese remakes of South Korean films": "x",
    "Category:Kannada remakes of Bengali films": "x",
    "Category:Kannada remakes of Hindi films": "x",
    "Category:Kannada remakes of Malayalam films": "x",
    "Category:Kannada remakes of Marathi films": "x",
    "Category:Kannada remakes of Rajasthani films": "x",
    "Category:Kannada remakes of Tamil films": "x",
    "Category:Kannada remakes of Telugu films": "x",
    "Category:Malayalam remakes of Bengali films": "x",
    "Category:Malayalam remakes of Hindi films": "x",
    "Category:Malayalam remakes of Kannada films": "x",
    "Category:Malayalam remakes of Marathi films": "x",
    "Category:Malayalam remakes of Tamil films": "x",
    "Category:Malayalam remakes of Telugu films": "x",
    "Category:Maldivian remakes of Indian films": "x",
    "Category:Marathi remakes of Bengali films": "x",
    "Category:Marathi remakes of Hindi films": "x",
    "Category:Marathi remakes of Kannada films": "x",
    "Category:Marathi remakes of Malayalam films": "x",
    "Category:Marathi remakes of Rajasthani films": "x",
    "Category:Marathi remakes of Tamil films": "x",
    "Category:Marathi remakes of Telugu films": "x",
    "Category:Meitei remakes of Telugu films": "x",
    "Category:Odia remakes of Bengali films": "x",
    "Category:Odia remakes of Hindi films": "x",
    "Category:Odia remakes of Kannada films": "x",
    "Category:Odia remakes of Malayalam films": "x",
    "Category:Odia remakes of Marathi films": "x",
    "Category:Odia remakes of Punjabi films": "x",
    "Category:Odia remakes of Tamil films": "x",
    "Category:Odia remakes of Telugu films": "x",
    "Category:Pakistani remakes of American films": "x",
    "Category:Pakistani remakes of foreign films": "x",
    "Category:Pakistani remakes of Indian films": "x",
    "Category:Philippine remakes of foreign films": "x",
    "Category:Philippine remakes of South Korean films": "x",
    "Category:Punjabi remakes of Hindi films": "x",
    "Category:Punjabi remakes of Malayalam films": "x",
    "Category:Punjabi remakes of Marathi films": "x",
    "Category:Punjabi remakes of Tamil films": "x",
    "Category:Punjabi remakes of Telugu films": "x",
    "Category:South Korean remakes of Brazilian films": "x",
    "Category:South Korean remakes of foreign films": "x",
    "Category:South Korean remakes of Japanese films": "x",
    "Category:South Korean remakes of Spanish films": "x",
    "Category:South Korean remakes of Taiwanese films": "x",
    "Category:Spanish remakes of Argentine films": "x",
    "Category:Spanish remakes of foreign films": "x",
    "Category:Spanish remakes of French films": "x",
    "Category:Spanish remakes of Italian films": "x",
    "Category:Sri Lankan remakes of Indian films": "x",
    "Category:Taiwanese remakes of foreign films": "x",
    "Category:Taiwanese remakes of South Korean films": "x",
    "Category:Tamil remakes of Bengali films": "x",
    "Category:Tamil remakes of Hindi films": "x",
    "Category:Tamil remakes of Kannada films": "x",
    "Category:Tamil remakes of Malayalam films": "x",
    "Category:Tamil remakes of Marathi films": "x",
    "Category:Tamil remakes of Telugu films": "x",
    "Category:Television remakes of films": "x",
    "Category:Telugu remakes of Bengali films": "x",
    "Category:Telugu remakes of Gujarati films": "x",
    "Category:Telugu remakes of Hindi films": "x",
    "Category:Telugu remakes of Kannada films": "x",
    "Category:Telugu remakes of Malayalam films": "x",
    "Category:Telugu remakes of Marathi films": "x",
    "Category:Telugu remakes of Punjabi films": "x",
    "Category:Telugu remakes of Tamil films": "x",
    "Category:Thai remakes of foreign films": "x",
    "Category:Thai remakes of Taiwanese films": "",
}


TEMPORAL_CASES = [
    ("test_Keep_it_last_extended", fast_data),
    # ("test_Keep_it_last_empty", fast_data_empty),
]


@pytest.mark.parametrize("name,data", TEMPORAL_CASES)
@pytest.mark.dump
def test_Keep_it_last_dump(name: str, data: dict[str, str]) -> None:
    expected, diff_result = one_dump_test(data, resolve_arabic_category_label)

    dump_diff(diff_result, name)
    assert diff_result == expected, f"Differences found: {len(diff_result):,}, len all :{len(data):,}"



#
import pytest
from load_one_data import dump_diff, dump_diff_text, one_dump_test

from ArWikiCats import resolve_arabic_category_label

data = {
    "Category:Gymnastics organizations": "تصنيف:منظمات جمباز",
    "Category:Publications by format": "تصنيف:منشورات حسب التنسيق",
    "Category:Publications disestablished in 1946": "تصنيف:منشورات انحلت في 1946",
    "Category:Subfields by academic discipline": "تصنيف:حقول فرعية حسب التخصص الأكاديمي",
    "Category:Women's organizations based in Cuba": "تصنيف:منظمات نسائية مقرها في كوبا",
}


@pytest.mark.parametrize("category, expected", data.items(), ids=data.keys())
@pytest.mark.fast
def test_institutions(category: str, expected: str) -> None:
    label = resolve_arabic_category_label(category)
    assert label == expected


data_2 = {
    "Category:Indonesian women singers by century": "تصنيف:مغنيات إندونيسيات حسب القرن",
    "Category:Iranian women singers by century": "تصنيف:مغنيات إيرانيات حسب القرن",
    "Category:20th-century Italian women singers": "تصنيف:مغنيات إيطاليات في القرن 20",
    "Category:Bulgarian women singers by century": "تصنيف:مغنيات بلغاريات حسب القرن",
    "Category:20th-century Panamanian women singers": "تصنيف:مغنيات بنميات في القرن 20",
    "Category:Puerto Rican women singers by century": "تصنيف:مغنيات بورتوريكيات حسب القرن",
    "Category:Women singers by former country": "تصنيف:مغنيات حسب البلد السابق",
    "Category:Women singers by ethnicity": "تصنيف:مغنيات حسب المجموعة العرقية",
    "Category:Women singers by genre": "تصنيف:مغنيات حسب النوع الفني",
    "Category:19th-century Sudanese women singers": "تصنيف:مغنيات سودانيات في القرن 19",
    "Category:Ghanaian women singers by century": "تصنيف:مغنيات غانيات حسب القرن",
    "Category:Finnish women singers by century": "تصنيف:مغنيات فنلنديات حسب القرن",
    "Category:17th-century women singers by nationality": "تصنيف:مغنيات في القرن 17 حسب الجنسية",
    "Category:18th-century women singers by nationality": "تصنيف:مغنيات في القرن 18 حسب الجنسية",
    "Category:Cuban women singers by century": "تصنيف:مغنيات كوبيات حسب القرن",
    "Category:20th-century Lithuanian women singers": "تصنيف:مغنيات ليتوانيات في القرن 20",
    "Category:19th-century Mexican women singers": "تصنيف:مغنيات مكسيكيات في القرن 19",
    "Category:Women singers from the Russian Empire": "تصنيف:مغنيات من الإمبراطورية الروسية",
    "Category:Women singers from the Holy Roman Empire": "تصنيف:مغنيات من الإمبراطورية الرومانية المقدسة",
    "Category:18th-century women singers from the Holy Roman Empire": "تصنيف:مغنيات من الإمبراطورية الرومانية المقدسة في القرن 18",
    "Category:Women singers from Georgia (country) by century": "تصنيف:مغنيات من جورجيا حسب القرن",
    "Category:Women singers from the Kingdom of Prussia": "تصنيف:مغنيات من مملكة بروسيا",
    "Category:Norwegian women singers by century": "تصنيف:مغنيات نرويجيات حسب القرن",
    "Category:Austrian women singers by century": "تصنيف:مغنيات نمساويات حسب القرن",
    "Category:Jewish women singers": "تصنيف:مغنيات يهوديات",
}


@pytest.mark.parametrize("category, expected", data_2.items(), ids=data_2.keys())
@pytest.mark.skip2
def test_women_singers(category: str, expected: str) -> None:
    label = resolve_arabic_category_label(category)
    assert label == expected


TEMPORAL_CASES = [
    ("test_institutions", data),
    ("test_women_singers", data_2),
]


@pytest.mark.parametrize("name,data", TEMPORAL_CASES)
@pytest.mark.dump
def test_all(name: str, data: dict[str, str]) -> None:
    expected, diff_result = one_dump_test(data, resolve_arabic_category_label)

    dump_diff(diff_result, name)
    # dump_diff_text(expected, diff_result, name)
    assert diff_result == expected, f"Differences found: {len(diff_result):,}, len all :{len(data):,}"



#!/usr/bin/python3
"""
NOTE: this file has alot of formatted_data

TODO: merge with sports_resolvers/nationalities_and_sports.py

"""

import functools
import re

from ...helps import len_print, logger
from ...new.handle_suffixes import resolve_sport_category_suffix_with_mapping
from ...translations import Nat_women
from ...translations.sports.Sport_key import SPORT_KEY_RECORDS
from ...translations_formats import format_multi_data_v2
from ...translations_formats.DataModel.model_multi_data import MultiDataFormatterBase
from ..nats_as_country_names import nats_keys_as_country_names

# TODO: add data from new_for_nat_female_xo_team_additional
new_for_nat_female_xo_team_2 = {
    # "yemeni football": "كرة قدم يمنية",
    # "{en} {en_sport}": "{sport_jobs} {female}",  # Category:American_basketball
    "{en} {en_sport} leagues": "دوريات {sport_jobs} {female}",
    "{en} {en_sport} competitions": "منافسات {sport_jobs} {female}",
    "{en_sport} competitions": "منافسات {sport_jobs}",
    # "{en} {en_sport}": "{sport_label} {the_female}",  # Category:American_basketball
    # "yemeni national football": "كرة قدم وطنية يمنية",
    "{en} national {en_sport}": "{sport_jobs} وطنية {female}",
    "{en} womens {en_sport}": "{sport_jobs} {female} نسائية",
    "amateur {en_sport} world cup": "كأس العالم {sport_team} للهواة",
    "mens {en_sport} world cup": "كأس العالم {sport_team} للرجال",
    "womens {en_sport} world cup": "كأس العالم {sport_team} للسيدات",
    "{en_sport} world cup": "كأس العالم {sport_team}",
    "youth {en_sport} world cup": "كأس العالم {sport_team} للشباب",
    "{en} amateur {en_sport} cup": "كأس {female} {sport_jobs} للهواة",
    "{en} youth {en_sport} cup": "كأس {female} {sport_jobs} للشباب",
    "{en} mens {en_sport} cup": "كأس {female} {sport_jobs} للرجال",
    "{en} womens {en_sport} cup": "كأس {female} {sport_jobs} للسيدات",
    "{en} defunct {en_sport} cup": "كؤوس {sport_jobs} {female} سابقة",
    "{en} {en_sport} cup": "كؤوس {sport_jobs} {female}",
    "{en} domestic {en_sport} cup": "كؤوس {sport_jobs} {female} محلية",
    "{en} {en_sport} teams": "فرق {sport_jobs} {female}",
    "{en} {en_sport} national teams": "منتخبات {sport_jobs} {female}",
    # tab[Category:Canadian domestic Soccer: "تصنيف:كرة قدم كندية محلية"
    "{en} domestic {en_sport}": "{sport_jobs} {female} محلية",
    "{en} domestic womens {en_sport}": "{sport_jobs} {female} محلية للسيدات",
    "{en} {en_sport} championships": "بطولات {sport_jobs} {female}",
    "{en} national {en_sport} championships": "بطولات {sport_jobs} وطنية {female}",
    # "{en} national {en_sport} champions": "أبطال بطولات {sport_jobs} وطنية {female}",
    "{en} national {en_sport} champions": "أبطال {sport_jobs} وطنية {female}",
    "{en} {en_sport} super leagues": "دوريات سوبر {sport_jobs} {female}",
    "{en} current {en_sport} seasons": "مواسم {sport_jobs} {female} حالية",
    # ---
    "{en} professional {en_sport}": "{sport_jobs} {female} للمحترفين",
    "{en} indoor {en_sport}": "{sport_jobs} {female} داخل الصالات",
    "{en} outdoor {en_sport}": "{sport_jobs} {female} في الهواء الطلق",
    "{en} defunct indoor {en_sport}": "{sport_jobs} {female} داخل الصالات سابقة",
    "{en} defunct outdoor {en_sport}": "{sport_jobs} {female} في الهواء الطلق سابقة",
    "{en} reserve {en_sport}": "{sport_jobs} {female} احتياطية",
    "{en} defunct {en_sport}": "{sport_jobs} {female} سابقة",
    # [european national womens volleyball teams] = "منتخبات كرة طائرة وطنية أوروبية للسيدات"
    "{en} national womens {en_sport} teams": "منتخبات {sport_jobs} وطنية {female} للسيدات",
    "{en} national {en_sport} teams": "منتخبات {sport_jobs} وطنية {female}",
    "{en} national a {en_sport} teams": "منتخبات {sport_jobs} محليين {female}",
    "{en} national b {en_sport} teams": "منتخبات {sport_jobs} رديفة {female}",
    "{en} national reserve {en_sport} teams": "منتخبات {sport_jobs} وطنية احتياطية {female}",
    "{en} national {en_sport} teams premier": "منتخبات {sport_jobs} وطنية {female} من الدرجة الممتازة",
    "{en} {en_sport} teams premier": "فرق {sport_jobs} {female} من الدرجة الممتازة",
}

new_for_nat_female_xo_team_additional = {
    "{en} national amateur under-13 {en_sport}": "{sport_jobs} وطنية {female} تحت 13 سنة للهواة",
    "{en} national amateur under-14 {en_sport}": "{sport_jobs} وطنية {female} تحت 14 سنة للهواة",
    "{en} national amateur under-15 {en_sport}": "{sport_jobs} وطنية {female} تحت 15 سنة للهواة",
    "{en} national amateur under-16 {en_sport}": "{sport_jobs} وطنية {female} تحت 16 سنة للهواة",
    "{en} national amateur under-17 {en_sport}": "{sport_jobs} وطنية {female} تحت 17 سنة للهواة",
    "{en} national amateur under-18 {en_sport}": "{sport_jobs} وطنية {female} تحت 18 سنة للهواة",
    "{en} national amateur under-19 {en_sport}": "{sport_jobs} وطنية {female} تحت 19 سنة للهواة",
    "{en} national amateur under-20 {en_sport}": "{sport_jobs} وطنية {female} تحت 20 سنة للهواة",
    "{en} national amateur under-21 {en_sport}": "{sport_jobs} وطنية {female} تحت 21 سنة للهواة",
    "{en} national amateur under-23 {en_sport}": "{sport_jobs} وطنية {female} تحت 23 سنة للهواة",
    "{en} national amateur under-24 {en_sport}": "{sport_jobs} وطنية {female} تحت 24 سنة للهواة",
    "{en} national amateur {en_sport}": "{sport_jobs} وطنية {female} للهواة",
    "{en} national junior mens under-13 {en_sport}": "{sport_jobs} وطنية {female} تحت 13 سنة للناشئين",
    "{en} national junior mens under-14 {en_sport}": "{sport_jobs} وطنية {female} تحت 14 سنة للناشئين",
    "{en} national junior mens under-15 {en_sport}": "{sport_jobs} وطنية {female} تحت 15 سنة للناشئين",
    "{en} national junior mens under-16 {en_sport}": "{sport_jobs} وطنية {female} تحت 16 سنة للناشئين",
    "{en} national junior mens under-17 {en_sport}": "{sport_jobs} وطنية {female} تحت 17 سنة للناشئين",
    "{en} national junior mens under-18 {en_sport}": "{sport_jobs} وطنية {female} تحت 18 سنة للناشئين",
    "{en} national junior mens under-19 {en_sport}": "{sport_jobs} وطنية {female} تحت 19 سنة للناشئين",
    "{en} national junior mens under-20 {en_sport}": "{sport_jobs} وطنية {female} تحت 20 سنة للناشئين",
    "{en} national junior mens under-21 {en_sport}": "{sport_jobs} وطنية {female} تحت 21 سنة للناشئين",
    "{en} national junior mens under-23 {en_sport}": "{sport_jobs} وطنية {female} تحت 23 سنة للناشئين",
    "{en} national junior mens under-24 {en_sport}": "{sport_jobs} وطنية {female} تحت 24 سنة للناشئين",
    "{en} national junior mens {en_sport}": "{sport_jobs} وطنية {female} للناشئين",
    "{en} national junior womens under-13 {en_sport}": "{sport_jobs} وطنية {female} تحت 13 سنة للناشئات",
    "{en} national junior womens under-14 {en_sport}": "{sport_jobs} وطنية {female} تحت 14 سنة للناشئات",
    "{en} national junior womens under-15 {en_sport}": "{sport_jobs} وطنية {female} تحت 15 سنة للناشئات",
    "{en} national junior womens under-16 {en_sport}": "{sport_jobs} وطنية {female} تحت 16 سنة للناشئات",
    "{en} national junior womens under-17 {en_sport}": "{sport_jobs} وطنية {female} تحت 17 سنة للناشئات",
    "{en} national junior womens under-18 {en_sport}": "{sport_jobs} وطنية {female} تحت 18 سنة للناشئات",
    "{en} national junior womens under-19 {en_sport}": "{sport_jobs} وطنية {female} تحت 19 سنة للناشئات",
    "{en} national junior womens under-20 {en_sport}": "{sport_jobs} وطنية {female} تحت 20 سنة للناشئات",
    "{en} national junior womens under-21 {en_sport}": "{sport_jobs} وطنية {female} تحت 21 سنة للناشئات",
    "{en} national junior womens under-23 {en_sport}": "{sport_jobs} وطنية {female} تحت 23 سنة للناشئات",
    "{en} national junior womens under-24 {en_sport}": "{sport_jobs} وطنية {female} تحت 24 سنة للناشئات",
    "{en} national junior womens {en_sport}": "{sport_jobs} وطنية {female} للناشئات",
    "{en} national mens under-13 {en_sport}": "{sport_jobs} وطنية {female} تحت 13 سنة للرجال",
    "{en} national mens under-14 {en_sport}": "{sport_jobs} وطنية {female} تحت 14 سنة للرجال",
    "{en} national mens under-15 {en_sport}": "{sport_jobs} وطنية {female} تحت 15 سنة للرجال",
    "{en} national mens under-16 {en_sport}": "{sport_jobs} وطنية {female} تحت 16 سنة للرجال",
    "{en} national mens under-17 {en_sport}": "{sport_jobs} وطنية {female} تحت 17 سنة للرجال",
    "{en} national mens under-18 {en_sport}": "{sport_jobs} وطنية {female} تحت 18 سنة للرجال",
    "{en} national mens under-19 {en_sport}": "{sport_jobs} وطنية {female} تحت 19 سنة للرجال",
    "{en} national mens under-20 {en_sport}": "{sport_jobs} وطنية {female} تحت 20 سنة للرجال",
    "{en} national mens under-21 {en_sport}": "{sport_jobs} وطنية {female} تحت 21 سنة للرجال",
    "{en} national mens under-23 {en_sport}": "{sport_jobs} وطنية {female} تحت 23 سنة للرجال",
    "{en} national mens under-24 {en_sport}": "{sport_jobs} وطنية {female} تحت 24 سنة للرجال",
    "{en} national mens {en_sport}": "{sport_jobs} وطنية {female} للرجال",
    "{en} national under-13 {en_sport}": "{sport_jobs} وطنية {female} تحت 13 سنة",
    "{en} national under-14 {en_sport}": "{sport_jobs} وطنية {female} تحت 14 سنة",
    "{en} national under-15 {en_sport}": "{sport_jobs} وطنية {female} تحت 15 سنة",
    "{en} national under-16 {en_sport}": "{sport_jobs} وطنية {female} تحت 16 سنة",
    "{en} national under-17 {en_sport}": "{sport_jobs} وطنية {female} تحت 17 سنة",
    "{en} national under-18 {en_sport}": "{sport_jobs} وطنية {female} تحت 18 سنة",
    "{en} national under-19 {en_sport}": "{sport_jobs} وطنية {female} تحت 19 سنة",
    "{en} national under-20 {en_sport}": "{sport_jobs} وطنية {female} تحت 20 سنة",
    "{en} national under-21 {en_sport}": "{sport_jobs} وطنية {female} تحت 21 سنة",
    "{en} national under-23 {en_sport}": "{sport_jobs} وطنية {female} تحت 23 سنة",
    "{en} national under-24 {en_sport}": "{sport_jobs} وطنية {female} تحت 24 سنة",
    "{en} national womens under-13 {en_sport}": "{sport_jobs} وطنية {female} تحت 13 سنة للسيدات",
    "{en} national womens under-14 {en_sport}": "{sport_jobs} وطنية {female} تحت 14 سنة للسيدات",
    "{en} national womens under-15 {en_sport}": "{sport_jobs} وطنية {female} تحت 15 سنة للسيدات",
    "{en} national womens under-16 {en_sport}": "{sport_jobs} وطنية {female} تحت 16 سنة للسيدات",
    "{en} national womens under-17 {en_sport}": "{sport_jobs} وطنية {female} تحت 17 سنة للسيدات",
    "{en} national womens under-18 {en_sport}": "{sport_jobs} وطنية {female} تحت 18 سنة للسيدات",
    "{en} national womens under-19 {en_sport}": "{sport_jobs} وطنية {female} تحت 19 سنة للسيدات",
    "{en} national womens under-20 {en_sport}": "{sport_jobs} وطنية {female} تحت 20 سنة للسيدات",
    "{en} national womens under-21 {en_sport}": "{sport_jobs} وطنية {female} تحت 21 سنة للسيدات",
    "{en} national womens under-23 {en_sport}": "{sport_jobs} وطنية {female} تحت 23 سنة للسيدات",
    "{en} national womens under-24 {en_sport}": "{sport_jobs} وطنية {female} تحت 24 سنة للسيدات",
    "{en} national womens {en_sport}": "{sport_jobs} وطنية {female} للسيدات",
    "{en} national youth under-13 {en_sport}": "{sport_jobs} وطنية {female} تحت 13 سنة للشباب",
    "{en} national youth under-14 {en_sport}": "{sport_jobs} وطنية {female} تحت 14 سنة للشباب",
    "{en} national youth under-15 {en_sport}": "{sport_jobs} وطنية {female} تحت 15 سنة للشباب",
    "{en} national youth under-16 {en_sport}": "{sport_jobs} وطنية {female} تحت 16 سنة للشباب",
    "{en} national youth under-17 {en_sport}": "{sport_jobs} وطنية {female} تحت 17 سنة للشباب",
    "{en} national youth under-18 {en_sport}": "{sport_jobs} وطنية {female} تحت 18 سنة للشباب",
    "{en} national youth under-19 {en_sport}": "{sport_jobs} وطنية {female} تحت 19 سنة للشباب",
    "{en} national youth under-20 {en_sport}": "{sport_jobs} وطنية {female} تحت 20 سنة للشباب",
    "{en} national youth under-21 {en_sport}": "{sport_jobs} وطنية {female} تحت 21 سنة للشباب",
    "{en} national youth under-23 {en_sport}": "{sport_jobs} وطنية {female} تحت 23 سنة للشباب",
    "{en} national youth under-24 {en_sport}": "{sport_jobs} وطنية {female} تحت 24 سنة للشباب",
    "{en} national youth womens under-13 {en_sport}": "{sport_jobs} وطنية {female} تحت 13 سنة للشابات",
    "{en} national youth womens under-14 {en_sport}": "{sport_jobs} وطنية {female} تحت 14 سنة للشابات",
    "{en} national youth womens under-15 {en_sport}": "{sport_jobs} وطنية {female} تحت 15 سنة للشابات",
    "{en} national youth womens under-16 {en_sport}": "{sport_jobs} وطنية {female} تحت 16 سنة للشابات",
    "{en} national youth womens under-17 {en_sport}": "{sport_jobs} وطنية {female} تحت 17 سنة للشابات",
    "{en} national youth womens under-18 {en_sport}": "{sport_jobs} وطنية {female} تحت 18 سنة للشابات",
    "{en} national youth womens under-19 {en_sport}": "{sport_jobs} وطنية {female} تحت 19 سنة للشابات",
    "{en} national youth womens under-20 {en_sport}": "{sport_jobs} وطنية {female} تحت 20 سنة للشابات",
    "{en} national youth womens under-21 {en_sport}": "{sport_jobs} وطنية {female} تحت 21 سنة للشابات",
    "{en} national youth womens under-23 {en_sport}": "{sport_jobs} وطنية {female} تحت 23 سنة للشابات",
    "{en} national youth womens under-24 {en_sport}": "{sport_jobs} وطنية {female} تحت 24 سنة للشابات",
    "{en} national youth womens {en_sport}": "{sport_jobs} وطنية {female} للشابات",
    "{en} national youth {en_sport}": "{sport_jobs} وطنية {female} للشباب",
    "{en} under-13 {en_sport}": "{sport_jobs} {female} تحت 13 سنة",
    "{en} under-14 {en_sport}": "{sport_jobs} {female} تحت 14 سنة",
    "{en} under-15 {en_sport}": "{sport_jobs} {female} تحت 15 سنة",
    "{en} under-16 {en_sport}": "{sport_jobs} {female} تحت 16 سنة",
    "{en} under-17 {en_sport}": "{sport_jobs} {female} تحت 17 سنة",
    "{en} under-18 {en_sport}": "{sport_jobs} {female} تحت 18 سنة",
    "{en} under-19 {en_sport}": "{sport_jobs} {female} تحت 19 سنة",
    "{en} under-20 {en_sport}": "{sport_jobs} {female} تحت 20 سنة",
    "{en} under-21 {en_sport}": "{sport_jobs} {female} تحت 21 سنة",
    "{en} under-23 {en_sport}": "{sport_jobs} {female} تحت 23 سنة",
    "{en} under-24 {en_sport}": "{sport_jobs} {female} تحت 24 سنة",
}

new_for_nat_female_xo_team_2.update(new_for_nat_female_xo_team_additional)


@functools.lru_cache(maxsize=1)
def _load_bot() -> MultiDataFormatterBase:
    nats_data = {x: {"female": v} for x, v in Nat_women.items()}
    sport_key_records = dict(SPORT_KEY_RECORDS)
    sport_key_records.get("sports", {}).update(
        {
            "jobs": "رياضية",
        }
    )
    nats_data.update({x: {"female": v.get("female")} for x, v in nats_keys_as_country_names.items() if v.get("female")})

    sports_data = {
        x: {
            "sport_label": v.get("label", ""),
            "sport_team": v.get("team", ""),
            "sport_jobs": v.get("jobs", ""),
        }
        for x, v in SPORT_KEY_RECORDS.items()
        if v.get("jobs")
    }

    return format_multi_data_v2(
        formatted_data=new_for_nat_female_xo_team_2,
        data_list=nats_data,
        key_placeholder="{en}",
        data_list2=sports_data,
        key2_placeholder="{en_sport}",
        text_after=" people",
        text_before="the ",
        search_first_part=True,
        use_other_formatted_data=True,
    )


@functools.lru_cache(maxsize=1)
def _load_end_key_mappings() -> dict[str, str]:
    keys_ending = {
        "teams": "فرق {lab}",
        "premier": "{lab} من الدرجة الممتازة",
        "first tier": "{lab} من الدرجة الأولى",
        "top tier": "{lab} من الدرجة الأولى",
        "second tier": "{lab} من الدرجة الثانية",
        "third tier": "{lab} من الدرجة الثالثة",
        "fourth tier": "{lab} من الدرجة الرابعة",
        "fifth tier": "{lab} من الدرجة الخامسة",
        "sixth tier": "{lab} من الدرجة السادسة",
        "seventh tier": "{lab} من الدرجة السابعة",
        "chairmen and investors": "رؤساء ومسيرو {lab}",
        "cups": "كؤوس {lab}",
        "champions": "أبطال {lab}",
        "clubs": "أندية {lab}",
        "clubs and teams": "أندية وفرق {lab}",
        "coaches": "مدربو {lab}",  # Category:Indoor soccer coaches in the United States by club
        "competitions": "منافسات {lab}",
        "events": "أحداث {lab}",
        "films": "أفلام {lab}",
        "finals": "نهائيات {lab}",
        "home stadiums": "ملاعب {lab}",
        "leagues": "دوريات {lab}",
        "lists": "قوائم {lab}",
        "manager history": "تاريخ مدربو {lab}",
        "managers": "مدربو {lab}",
        "matches": "مباريات {lab}",
        "navigational boxes": "صناديق تصفح {lab}",
        "non-profit organizations": "منظمات غير ربحية {lab}",
        "non-profit publishers": "ناشرون غير ربحيون {lab}",
        "organisations": "منظمات {lab}",
        "organizations": "منظمات {lab}",
        "players": "لاعبو {lab}",
        "positions": "مراكز {lab}",
        "records": "سجلات {lab}",
        "records and statistics": "سجلات وإحصائيات {lab}",
        "results": "نتائج {lab}",
        "rivalries": "دربيات {lab}",
        "scouts": "كشافة {lab}",
        "squads": "تشكيلات {lab}",
        "statistics": "إحصائيات {lab}",
        "templates": "قوالب {lab}",
        "tournaments": "بطولات {lab}",
        "trainers": "مدربو {lab}",
        "umpires": "حكام {lab}",
        "venues": "ملاعب {lab}",
    }

    keys_ending = dict(
        sorted(
            keys_ending.items(),
            key=lambda k: (-k[0].count(" "), -len(k[0])),
        )
    )
    return keys_ending


@functools.lru_cache(maxsize=None)
def _sport_lab_nat_load_new(category) -> str:
    logger.debug(f"<<yellow>> start _sport_lab_nat_load_new: {category=}")
    both_bot = _load_bot()
    result = both_bot.search_all_category(category)
    logger.info_if_or_debug(f"<<yellow>> end _sport_lab_nat_load_new: {category=}, {result=}", result)
    return result


def fix_keys(category: str) -> str:
    category = category.replace("'", "").lower()

    replacements = {
        "level": "tier",
        "canadian football": "canadian-football",
    }

    for old, new in replacements.items():
        category = re.sub(rf"\b{re.escape(old)}\b", new, category)

    return category


def fix_result_callable(result: str, category: str, key: str, value: str) -> str:
    if result.startswith("لاعبو ") and "للسيدات" in result:
        result = result.replace("لاعبو ", "لاعبات ")

    if key == "teams" and "national" in category:
        result = result.replace("فرق ", "منتخبات ")

    return result


@functools.lru_cache(maxsize=10000)
def sport_lab_nat_load_new(category: str) -> str:
    category = fix_keys(category)

    logger.debug(f"<<yellow>> start sport_lab_nat_load_new: {category=}")
    keys_ending = _load_end_key_mappings()

    result = resolve_sport_category_suffix_with_mapping(
        category=category,
        data=keys_ending,
        callback=_sport_lab_nat_load_new,
        fix_result_callable=fix_result_callable,
        format_key="lab",
    )

    logger.info_if_or_debug(f"<<yellow>> end sport_lab_nat_load_new: {category=}, {result=}", result)
    return result


len_print.data_len(
    "sports_formats_teams/sport_lab_nat.py",
    {
        "new_for_nat_female_xo_team_2": new_for_nat_female_xo_team_2,
        "new_for_nat_female_xo_team_additional": new_for_nat_female_xo_team_additional,
    },
)

__all__ = [
    "sport_lab_nat_load_new",
]



#
import pytest

from ArWikiCats import resolve_arabic_category_label

data = {
    "Category:Egyptian oncologists": "تصنيف:أطباء أورام مصريون",
    "Category:Fish described in 1995": "تصنيف:أسماك وصفت في 1995",
    "Category:Mammals described in 2017": "تصنيف:ثدييات وصفت في 2017",
    "Category:Pakistani psychiatrists": "تصنيف:أطباء نفسيون باكستانيون",
    "Category:Research institutes established in 1900": "تصنيف:معاهد أبحاث أسست في 1900",
    "Category:Swedish oncologists": "تصنيف:أطباء أورام سويديون",
}


@pytest.mark.parametrize("category, expected", data.items(), ids=data.keys())
@pytest.mark.fast
def test_science_and_medicine(category: str, expected: str) -> None:
    label = resolve_arabic_category_label(category)
    assert label == expected



#!/usr/bin/python3
"""
Module for handling year-and-from based category translations.

This module provides classes for formatting template-driven translation labels
that combine temporal patterns (years, decades, centuries) with "from" relation
patterns (e.g., "writers from Yemen", "people from Germany").

The module integrates with category_relation_mapping to resolve relation words
(prepositions like "from", "in", "by") into their Arabic equivalents.

Classes:
    FormatDataFrom: A dynamic wrapper for handling category transformations
        with customizable callbacks for key matching and searching.
    MultiDataFormatterYearAndFrom: Combines year-based and "from" relation
        category translations using the parent class helpers.

Example:
    >>> from ArWikiCats.translations_formats import MultiDataFormatterYearAndFrom, FormatDataFrom
    >>> country_bot = FormatDataFrom(
    ...     formatted_data={"{year1} {country1}": "{country1} في {year1}"},
    ...     key_placeholder="{country1}",
    ...     value_placeholder="{country1}",
    ...     search_callback=get_label_func,
    ...     match_key_callback=match_key_func,
    ... )
    >>> year_bot = FormatDataFrom(
    ...     formatted_data={},
    ...     key_placeholder="{year1}",
    ...     value_placeholder="{year1}",
    ...     search_callback=convert_time_to_arabic,
    ...     match_key_callback=match_time_en_first,
    ... )
    >>> bot = MultiDataFormatterYearAndFrom(country_bot, year_bot, other_key_first=True)
    >>> bot.create_label("14th-century writers from Yemen")
    'كتاب من اليمن في القرن 14'
"""
from .model_multi_data_base import MultiDataFormatterBaseHelpers
from .model_multi_data_year_from import FormatDataFrom


class MultiDataFormatterYearAndFrom2(MultiDataFormatterBaseHelpers):
    """
    Combines year-based and "from" relation category translations.

    This class orchestrates two FormatDataFrom instances (country_bot and year_bot)
    to normalize and translate category strings that contain both temporal patterns
    and "from" relation patterns.

    The class integrates with category_relation_mapping to resolve relation words
    (prepositions like "from", "in", "by") into their Arabic equivalents when
    building labels.

    Attributes:
        country_bot (FormatDataFrom): Handles the "from" relation part (e.g., "writers from Yemen").
        other_bot (FormatDataFrom): Handles the year/time part (e.g., "14th-century").
        search_first_part (bool): If True, search using only the first part (after country normalization).
        data_to_find (dict[str, str] | None): Optional direct lookup dictionary for category labels.
        other_key_first (bool): If True, process the year/other key before the country key.

    Example:
        >>> bot = MultiDataFormatterYearAndFrom(country_bot, year_bot, other_key_first=True)
        >>> bot.create_label("14th-century writers from Yemen")
        'كتاب من اليمن في القرن 14'

        >>> bot.get_relation_word("People from Germany")
        ('from', 'من')

        >>> bot.resolve_relation_label("People from Germany", "أشخاص")
        'أشخاص من'
    """

    def __init__(
        self,
        country_bot: FormatDataFrom,
        year_bot: FormatDataFrom,
        category_relation_mapping: dict[str, str] = {},
        search_first_part: bool = False,
        data_to_find: dict[str, str] | None = None,
        other_key_first: bool = False,
    ) -> None:
        """Prepare helpers for matching and formatting template-driven labels.

        Args:
            country_bot: FormatDataFrom instance for handling "from" relation patterns.
            year_bot: FormatDataFrom instance for handling year/time patterns.
            category_relation_mapping: Mapping of relation words to Arabic translations.
            search_first_part: If True, search using only the first part after normalization.
            data_to_find: Optional dictionary for direct category-to-label lookups.
            other_key_first: If True, process year/other key before country key.
        """
        self.search_first_part = search_first_part
        self.country_bot = country_bot
        self.other_bot = year_bot
        self.data_to_find = data_to_find
        self.other_key_first = other_key_first

        self.category_relation_mapping = dict(
            sorted(
                category_relation_mapping.items(),
                key=lambda k: (-k[0].count(" "), -len(k[0])),
            )
        )

    def get_relation_word(self, category: str) -> tuple[str, str]:
        """
        Find the first relation word in the category using category_relation_mapping.

        Searches for relation words (prepositions like "from", "in", "by") in the
        category string and returns the matching key and its Arabic translation.

        Note:
            The method returns the first match found based on the iteration order
            of category_relation_mapping dictionary (insertion order in Python 3.7+).
            Longer/more specific relation phrases should be defined before shorter ones
            in the mapping to ensure correct matching (e.g., "published by" before "by").

        Args:
            category: The English category string to search.

        Returns:
            A tuple of (relation_key, arabic_translation). Returns ("", "") if no match.

        Example:
            >>> bot.get_relation_word("People from Germany")
            ('from', 'من')
            >>> bot.get_relation_word("Buildings in France")
            ('in', 'في')
            >>> bot.get_relation_word("Works published by Oxford")
            ('published by', 'نشرتها')
        """
        for separator, separator_name in self.category_relation_mapping.items():
            separator_with_spaces = f" {separator} "
            if separator_with_spaces in category:
                return separator, separator_name
        return "", ""

    def resolve_relation_label(self, category: str, base_label: str) -> str:
        """
        Append the Arabic relation word to a base label based on the category.

        Finds the relation word in the category and appends its Arabic equivalent
        to the base label if appropriate.

        Args:
            category: The English category string containing a relation word.
            base_label: The Arabic base label to append the relation to.

        Returns:
            The base label with the Arabic relation word appended, or the original
            base label if no relation is found or if it's already present.

        Example:
            >>> bot.resolve_relation_label("Writers from Yemen", "كتاب")
            'كتاب من'
            >>> bot.resolve_relation_label("People in Germany", "أشخاص")
            'أشخاص في'
        """
        if not base_label or not category:
            return base_label

        relation_key, relation_ar = self.get_relation_word(category)

        if not relation_ar:
            return base_label

        # Avoid duplicate relation words by checking if it ends with the relation
        # or contains it as a complete word (surrounded by spaces or at boundaries)
        relation_ar_stripped = relation_ar.strip()
        if base_label.endswith(relation_ar_stripped) or base_label.endswith(f" {relation_ar_stripped}"):
            return base_label
        if f" {relation_ar_stripped} " in base_label:
            return base_label

        return f"{base_label} {relation_ar}".strip()

    def get_relation_mapping(self) -> dict[str, str]:
        """
        Return the category_relation_mapping dictionary.

        This provides access to the full relation word mapping for external use
        or inspection.

        Returns:
            The category_relation_mapping dictionary with English keys and Arabic values.

        Example:
            >>> mapping = bot.get_relation_mapping()
            >>> mapping["from"]
            'من'
            >>> mapping["published by"]
            'نشرتها'
        """
        return self.category_relation_mapping



#
import pytest
from load_one_data import dump_diff, dump_diff_text, one_dump_test

from ArWikiCats import resolve_label_ar

data0 = {
    "North African American culture": "",
    "North African Cup of Champions": "",
    "North African Futsal Tournament": "",
    "North African campaign": "",
    "North African campaign films": "",
    "North African football biography stubs": "",
    "North African legendary creatures": "",
}

data1 = {}

data_2 = {
    "American people of North African descent": "أمريكيون من أصل إفريقي شمالي",
    "Argentine people of North African descent": "أرجنتينيون من أصل إفريقي شمالي",
    "Australian people of North African descent": "أستراليون من أصل إفريقي شمالي",
    "Chinese people of North African descent": "صينيون من أصل إفريقي شمالي",
    "Egyptian people of North African descent": "مصريون من أصل إفريقي شمالي",
    "Emirati people of North African descent": "إماراتيون من أصل إفريقي شمالي",
    "European people of North African descent": "أوروبيون من أصل إفريقي شمالي",
    "Filipino people of North African descent": "فلبينيون من أصل إفريقي شمالي",
    "French people of North African descent": "فرنسيون من أصل إفريقي شمالي",
    "Hong Kong people of North African descent": "هونغ كونغيون من أصل إفريقي شمالي",
    "Indian people of North African descent": "هنود من أصل إفريقي شمالي",
    "Indonesian people of North African descent": "إندونيسيون من أصل إفريقي شمالي",
    "Israeli people of North African descent": "إسرائيليون من أصل إفريقي شمالي",
    "Japanese people of North African descent": "يابانيون من أصل إفريقي شمالي",
    "Lebanese people of North African descent": "لبنانيون من أصل إفريقي شمالي",
    "Nigerian people of North African descent": "نيجيريون من أصل إفريقي شمالي",
    "North African Super Cup": "كأس السوبر الإفريقي الشمالي",
    "North African art": "فن إفريقي شمالي",
    "North African cuisine": "مطبخ إفريقي شمالي",
    "North African diaspora": "شتات إفريقي شمالي",
    "North African diaspora in Canada": "شتات إفريقي شمالي في كندا",
    "North African diaspora in France": "شتات إفريقي شمالي في فرنسا",
    "North African diaspora in Israel": "شتات إفريقي شمالي في إسرائيل",
    "North African diaspora in North America": "شتات إفريقي شمالي في أمريكا الشمالية",
    "North African diaspora in Paris": "شتات إفريقي شمالي في باريس",
    "North African diaspora in the United States": "شتات إفريقي شمالي في الولايات المتحدة",
    "North African musical instruments": "آلات موسيقية إفريقية شمالية",
    "North African people": "إفريقيون شماليون",
    "People of North African descent": "أشخاص من أصل إفريقي شمالي",
    "Tunisian people of North African descent": "تونسيون من أصل إفريقي شمالي",
}
data_3 = {}

to_test = [
    ("test_fix_nationalities_data_2", data_2),
    # ("test_fix_nationalities_data_1", data1),
    # ("test_fix_nationalities_data_3", data_3),
]


@pytest.mark.parametrize("category, expected", data_2.items(), ids=data_2.keys())
def test_fix_nationalities_data_2(category: str, expected: str) -> None:
    assert resolve_label_ar(category) == expected


@pytest.mark.parametrize("name,data", to_test)
@pytest.mark.dump
def test_dump_it(name: str, data: dict[str, str]) -> None:
    expected, diff_result = one_dump_test(data, resolve_label_ar)
    dump_diff(diff_result, name)

    # dump_diff_text (expected, diff_result, name)
    assert diff_result == expected, f"Differences found: {len(diff_result):,}, len all :{len(data):,}"



"""
Tests
"""

import pytest

from ArWikiCats.legacy_bots.o_bots.univer import te_universities

te_universities_data = {
    "seoul national university": "جامعة سول الوطنية",
    "i̇zmir university of economics": "جامعة إزمير للاقتصاد",
    "shandong university of science and technology": "جامعة شاندونغ للعلوم والتكنولوجيا",
    "kano university of science and technology": "جامعة كانو للعلوم والتكنولوجيا",
    "kaunas university-of-technology": "جامعة كاوناس للتكنولوجيا",
    "shanghai university-of-technology": "جامعة شانغهاي للتكنولوجيا",
    "beijing university-of-technology": "جامعة بكين للتكنولوجيا",
    "kharkiv national university": "جامعة خاركيف الوطنية",
    "kraków university of economics": "جامعة كراكوف للاقتصاد",
    "lille university of science and technology": "جامعة مدينة ليل للعلوم والتكنولوجيا",
    "luleå university-of-technology": "جامعة لوليو للتكنولوجيا",
    "kunming university of science and technology": "جامعة كونمينغ للعلوم والتكنولوجيا",
    "kyoto university of arts": "جامعة كيوتو للفنون",
    "munich university of applied sciences": "جامعة ميونخ للعلوم التطبيقية",
    "lanzhou university-of-technology": "جامعة لانتشو للتكنولوجيا",
    "lappeenranta university-of-technology": "جامعة لابينرنتا للتكنولوجيا",
    "wuhan university-of-technology": "جامعة ووهان للتكنولوجيا",
    "chengdu university-of-technology": "جامعة تشنغدو للتكنولوجيا",
    "xi'an university of science and technology": "جامعة شيان للعلوم والتكنولوجيا",
    "xi'an university-of-technology": "جامعة شيان للتكنولوجيا",
    "tabriz university of medical sciences": "جامعة تبريز للعلوم الطبية",
    "tainan national university": "جامعة تاينان الوطنية",
    "chinhoyi university-of-technology": "جامعة تشينهوي للتكنولوجيا",
    "taipei national university": "جامعة تايبيه الوطنية",
    "taiyuan university-of-technology": "جامعة تاي يوان للتكنولوجيا",
    "yokohama national university": "جامعة يوكوهاما الوطنية",
    "zhejiang university-of-technology": "جامعة تشيجيانغ للتكنولوجيا",
    "macau university of science and technology": "جامعة ماكاو للعلوم والتكنولوجيا",
    "tehran university of art": "جامعة طهران للفنون",
    "tehran university of medical sciences": "جامعة طهران للعلوم الطبية",
    "dalian university-of-technology": "جامعة داليان للتكنولوجيا",
    "university of science and technology beijing": "جامعة بكين للعلوم والتكنولوجيا",
    "mashhad university of medical sciences": "جامعة مشهد للعلوم الطبية",
    "donetsk national university": "جامعة دونيتسك الوطنية",
    "university-of-technology sydney": "جامعة سيدني للتكنولوجيا",
    "auckland university-of-technology": "جامعة أوكلاند للتكنولوجيا",
    "nanjing university of science and technology": "جامعة نانجينغ للعلوم والتكنولوجيا",
    "nanjing university of arts": "جامعة نانجينغ للفنون",
    "poznań university-of-technology": "جامعة بوزنان للتكنولوجيا",
    "berlin university of arts": "جامعة برلين للفنون",
    "bern university of applied sciences": "جامعة برن للعلوم التطبيقية",
    "braunschweig university of art": "جامعة براونشفايغ للفنون",
    "brno university-of-technology": "جامعة برنو للتكنولوجيا",
    "bucharest national university": "جامعة بوخارست الوطنية",
    "budapest university-of-technology": "جامعة بودابست للتكنولوجيا",
    "warsaw university-of-technology": "جامعة وارسو للتكنولوجيا",
    "osaka university of arts": "جامعة أوساكا للفنون",
    "wrocław university of economics": "جامعة فروتسواف للاقتصاد",
    "wrocław university of science and technology": "جامعة فروتسواف للعلوم والتكنولوجيا",
    "guilin university-of-technology": "جامعة غويلين للتكنولوجيا",
    "zurich university of arts": "جامعة زيورخ للفنون",
    "tianjin university-of-technology": "جامعة تيانجين للتكنولوجيا",
    "hanoi national university": "جامعة هانوي الوطنية",
    "hanoi university of science and technology": "جامعة هانوي للعلوم والتكنولوجيا",
    "tokyo university of science": "جامعة طوكيو للعلوم",
    "tokyo university of arts": "جامعة طوكيو للفنون",
    "hefei university-of-technology": "جامعة خفي للتكنولوجيا",
    "pohang university of science and technology": "جامعة بوهانغ للعلوم والتكنولوجيا",
    "szczecin university-of-technology": "جامعة شتتين للتكنولوجيا",
    "tallinn university-of-technology": "جامعة تالين للتكنولوجيا",
    "delft university-of-technology": "جامعة دلفت للتكنولوجيا",
    "eindhoven university-of-technology": "جامعة أيندهوفن للتكنولوجيا",
    "incheon national university": "جامعة إنتشون الوطنية",
    "isfahan university of medical sciences": "جامعة أصفهان للعلوم الطبية",
    "isfahan university-of-technology": "جامعة أصفهان للتكنولوجيا",
    "frankfurt university of applied sciences": "جامعة فرانكفورت للعلوم التطبيقية",
}


@pytest.mark.parametrize("category, expected_key", te_universities_data.items(), ids=te_universities_data.keys())
@pytest.mark.fast
def test_universities_data(category: str, expected_key: str) -> None:
    label = te_universities(category)
    assert label == expected_key


def test_te_universities() -> None:
    # Test with a basic university category
    result = te_universities("university of california")
    assert isinstance(result, str)

    result_empty = te_universities("")
    assert isinstance(result_empty, str)

    # Test with a specific major
    result_major = te_universities("university of engineering")
    assert isinstance(result_major, str)

    # Test with "the" prefix
    result_the = te_universities("the university of law")
    assert isinstance(result_the, str)



"""Unit tests"""

import pytest
from load_one_data import dump_diff, one_dump_test

from ArWikiCats import resolve_arabic_category_label

wheelchair_racers_by_nat = {
    "Category:American men wheelchair racers": "تصنيف:متسابقو كراسي متحركة أمريكيون",
    "Category:Australian men wheelchair racers": "تصنيف:متسابقو كراسي متحركة أستراليون",
    "Category:Austrian men wheelchair racers": "تصنيف:متسابقو كراسي متحركة نمساويون",
    "Category:Belgian men wheelchair racers": "تصنيف:متسابقو كراسي متحركة بلجيكيون",
    "Category:Brazilian men wheelchair racers": "تصنيف:متسابقو كراسي متحركة برازيليون",
    "Category:British men wheelchair racers": "تصنيف:متسابقو كراسي متحركة بريطانيون",
    "Category:Canadian men wheelchair racers": "تصنيف:متسابقو كراسي متحركة كنديون",
    "Category:Chinese men wheelchair racers": "تصنيف:متسابقو كراسي متحركة صينيون",
    "Category:Dutch men wheelchair racers": "تصنيف:متسابقو كراسي متحركة هولنديون",
    "Category:English men wheelchair racers": "تصنيف:متسابقو كراسي متحركة إنجليز",
    "Category:Finnish men wheelchair racers": "تصنيف:متسابقو كراسي متحركة فنلنديون",
    "Category:French men wheelchair racers": "تصنيف:متسابقو كراسي متحركة فرنسيون",
    "Category:Gabonese men wheelchair racers": "تصنيف:متسابقو كراسي متحركة غابونيون",
    "Category:German men wheelchair racers": "تصنيف:متسابقو كراسي متحركة ألمان",
    "Category:Irish men wheelchair racers": "تصنيف:متسابقو كراسي متحركة أيرلنديون",
    "Category:Israeli men wheelchair racers": "تصنيف:متسابقو كراسي متحركة إسرائيليون",
    "Category:Japanese men wheelchair racers": "تصنيف:متسابقو كراسي متحركة يابانيون",
    "Category:Mexican men wheelchair racers": "تصنيف:متسابقو كراسي متحركة مكسيكيون",
    "Category:Swiss men wheelchair racers": "تصنيف:متسابقو كراسي متحركة سويسريون",
    "Category:Welsh men wheelchair racers": "تصنيف:متسابقو كراسي متحركة ويلزيون",
    "Category:American wheelchair racers": "تصنيف:متسابقو كراسي متحركة أمريكيون",
    "Category:American women wheelchair racers": "تصنيف:متسابقات كراسي متحركة أمريكيات",
    "Category:Australian wheelchair racers": "تصنيف:متسابقو كراسي متحركة أستراليون",
    "Category:Australian women wheelchair racers": "تصنيف:متسابقات كراسي متحركة أستراليات",
    "Category:Austrian wheelchair racers": "تصنيف:متسابقو كراسي متحركة نمساويون",
    "Category:Belgian wheelchair racers": "تصنيف:متسابقو كراسي متحركة بلجيكيون",
    "Category:Belgian women wheelchair racers": "تصنيف:متسابقات كراسي متحركة بلجيكيات",
    "Category:Brazilian wheelchair racers": "تصنيف:متسابقو كراسي متحركة برازيليون",
    "Category:Brazilian women wheelchair racers": "تصنيف:متسابقات كراسي متحركة برازيليات",
    "Category:British wheelchair racers": "تصنيف:متسابقو كراسي متحركة بريطانيون",
    "Category:British women wheelchair racers": "تصنيف:متسابقات كراسي متحركة بريطانيات",
    "Category:Canadian wheelchair racers": "تصنيف:متسابقو كراسي متحركة كنديون",
    "Category:Canadian women wheelchair racers": "تصنيف:متسابقات كراسي متحركة كنديات",
    "Category:Chinese wheelchair racers": "تصنيف:متسابقو كراسي متحركة صينيون",
    "Category:Chinese women wheelchair racers": "تصنيف:متسابقات كراسي متحركة صينيات",
    "Category:Czech wheelchair racers": "تصنيف:متسابقو كراسي متحركة تشيكيون",
    "Category:Danish wheelchair racers": "تصنيف:متسابقو كراسي متحركة دنماركيون",
    "Category:Dutch wheelchair racers": "تصنيف:متسابقو كراسي متحركة هولنديون",
    "Category:Dutch women wheelchair racers": "تصنيف:متسابقات كراسي متحركة هولنديات",
    "Category:Emirati wheelchair racers": "تصنيف:متسابقو كراسي متحركة إماراتيون",
    "Category:English wheelchair racers": "تصنيف:متسابقو كراسي متحركة إنجليز",
    "Category:English women wheelchair racers": "تصنيف:متسابقات كراسي متحركة إنجليزيات",
    "Category:Finnish wheelchair racers": "تصنيف:متسابقو كراسي متحركة فنلنديون",
    "Category:Finnish women wheelchair racers": "تصنيف:متسابقات كراسي متحركة فنلنديات",
    "Category:French wheelchair racers": "تصنيف:متسابقو كراسي متحركة فرنسيون",
    "Category:Gabonese wheelchair racers": "تصنيف:متسابقو كراسي متحركة غابونيون",
    "Category:German wheelchair racers": "تصنيف:متسابقو كراسي متحركة ألمان",
    "Category:Irish wheelchair racers": "تصنيف:متسابقو كراسي متحركة أيرلنديون",
    "Category:Irish women wheelchair racers": "تصنيف:متسابقات كراسي متحركة أيرلنديات",
    "Category:Israeli wheelchair racers": "تصنيف:متسابقو كراسي متحركة إسرائيليون",
    "Category:Italian wheelchair racers": "تصنيف:متسابقو كراسي متحركة إيطاليون",
    "Category:Japanese wheelchair racers": "تصنيف:متسابقو كراسي متحركة يابانيون",
    "Category:Japanese women wheelchair racers": "تصنيف:متسابقات كراسي متحركة يابانيات",
    "Category:Kuwaiti wheelchair racers": "تصنيف:متسابقو كراسي متحركة كويتيون",
    "Category:Lithuanian wheelchair racers": "تصنيف:متسابقو كراسي متحركة ليتوانيون",
    "Category:Macedonian wheelchair racers": "تصنيف:متسابقو كراسي متحركة مقدونيون",
    "Category:Men wheelchair racers": "تصنيف:متسابقو كراسي متحركة",
    "Category:Mexican wheelchair racers": "تصنيف:متسابقو كراسي متحركة مكسيكيون",
    "Category:Mexican women wheelchair racers": "تصنيف:متسابقات كراسي متحركة مكسيكيات",
    "Category:Norwegian wheelchair racers": "تصنيف:متسابقو كراسي متحركة نرويجيون",
    "Category:Russian wheelchair racers": "تصنيف:متسابقو كراسي متحركة روس",
    "Category:Paralympic wheelchair racers": "تصنيف:متسابقو كراسي متحركة في الألعاب البارالمبية",
    "Category:Polish wheelchair racers": "تصنيف:متسابقو كراسي متحركة بولنديون",
    "Category:Sammarinese wheelchair racers": "تصنيف:متسابقو كراسي متحركة سان مارينيون",
    "Category:Scottish wheelchair racers": "تصنيف:متسابقو كراسي متحركة إسكتلنديون",
    "Category:Scottish women wheelchair racers": "تصنيف:متسابقات كراسي متحركة إسكتلنديات",
    "Category:South Korean wheelchair racers": "تصنيف:متسابقو كراسي متحركة كوريون جنوبيون",
    "Category:Spanish wheelchair racers": "تصنيف:متسابقو كراسي متحركة إسبان",
    "Category:Swedish wheelchair racers": "تصنيف:متسابقو كراسي متحركة سويديون",
    "Category:Swiss wheelchair racers": "تصنيف:متسابقو كراسي متحركة سويسريون",
    "Category:Swiss women wheelchair racers": "تصنيف:متسابقات كراسي متحركة سويسريات",
    "Category:Thai wheelchair racers": "تصنيف:متسابقو كراسي متحركة تايلنديون",
    "Category:Tunisian wheelchair racers": "تصنيف:متسابقو كراسي متحركة تونسيون",
    "Category:Turkish wheelchair racers": "تصنيف:متسابقو كراسي متحركة أتراك",
    "Category:Turkish women wheelchair racers": "تصنيف:متسابقات كراسي متحركة تركيات",
    "Category:Welsh wheelchair racers": "تصنيف:متسابقو كراسي متحركة ويلزيون",
    "Category:Welsh women wheelchair racers": "تصنيف:متسابقات كراسي متحركة ويلزيات",
    "Category:Wheelchair racers at the 2020 Summer Olympics": "تصنيف:متسابقو كراسي متحركة في الألعاب الأولمبية الصيفية 2020",
    "Category:Wheelchair racers by nationality": "تصنيف:متسابقو كراسي متحركة حسب الجنسية",
    "Category:Wheelchair racers": "تصنيف:متسابقو كراسي متحركة",
    "Category:Women wheelchair racers": "تصنيف:متسابقات كراسي متحركة",
    "Category:Zambian wheelchair racers": "تصنيف:متسابقو كراسي متحركة زامبيون",
    "Category:Australia women's national wheelchair basketball team": "تصنيف:منتخب أستراليا لكرة السلة على الكراسي المتحركة للسيدات",
    "Category:Women's National Wheelchair Basketball League": "تصنيف:الدوري الوطني لكرة السلة على الكراسي المتحركة للسيدات",
    "Category:New Zealand wheelchair racers": "تصنيف:متسابقو كراسي متحركة نيوزيلنديون",
    "Category:New Zealand wheelchair rugby players": "تصنيف:لاعبو رجبي على كراسي متحركة نيوزيلنديون",
    "Category:Olympic men wheelchair racers": "تصنيف:متسابقو كراسي متحركة في الألعاب الأولمبية",
    "Category:Olympic women wheelchair racers": "تصنيف:متسابقات كراسي متحركة في الألعاب الأولمبية",
}


data2 = {
    "Category:European Wheelchair Handball Nations’ Tournament": "",
    "Category:French Open by year – Wheelchair events": "",
    "Category:College men's wheelchair basketball teams in the United States": "",
    "Category:College women's wheelchair basketball teams in the United States": "",
    "Category:Canadian wheelchair sports competitors": "",
    "Category:British wheelchair shot putters": "",
    "Category:British wheelchair sports competitors": "",
    "Category:British wheelchair track and field athletes": "",
    "Category:American wheelchair javelin throwers": "",
    "Category:American wheelchair shot putters": "",
    "Category:American wheelchair sports competitors": "",
    "Category:American wheelchair track and field athletes": "",
    "Category:Australian Open by year – Wheelchair events": "",
    "Category:IWBF U23 World Wheelchair Basketball Championship": "",
    "Category:Pan American Wheelchair Handball Championship": "",
    "Category:Paralympic wheelchair basketball squads": "",
    "Category:Puerto Rican wheelchair sports competitors": "",
    "Category:Puerto Rican wheelchair track and field athletes": "",
    "Category:RFL Wheelchair Super League": "",
    "Category:Women's U25 Wheelchair Basketball World Championship": "",
    "Category:World Wheelchair Mixed Doubles Curling Championship": "",
    "Category:Wheelchair rugby Paralympic champions navigational boxes": "",
    "Category:Wheelchair Tennis Masters": "",
    "Category:Wheelchair basketball at the 2020 ASEAN Para Games": "",
    "Category:Wheelchair basketball at the ASEAN Para Games": "",
    "Category:Wheelchair fencing at the 2020 ASEAN Para Games": "",
    "Category:Wheelchair fencing at the ASEAN Para Games": "",
    "Category:Wheelchair javelin throwers": "",
    "Category:Wheelchair manufacturers": "",
    "Category:Wheelchair marathons": "",
    "Category:Wheelchair organizations": "",
    "Category:Wheelchair rugby biography stubs": "",
    "Category:Wheelchair shot putters": "",
    "Category:Wheelchair sports classifications": "",
    "Category:Wheelchair sports competitors by nationality": "",
    "Category:Wheelchair sports competitors": "",
    "Category:Wheelchair tennis at the 2020 ASEAN Para Games": "",
    "Category:Wheelchair tennis at the ASEAN Para Games": "",
    "Category:Wheelchair track and field athletes by nationality": "",
    "Category:Wheelchair track and field athletes": "",
    "Category:Wheelchair users by nationality": "",
    "Category:Wheelchair users from Georgia (country)": "",
    "Category:Wheelchair-category Paralympic competitors": "",
    "Category:Wheelchairs": "",
    "Category:Wimbledon Championship by year – Wheelchair events": "",
    "Category:Wimbledon Championship by year – Wheelchair men's doubles": "",
    "Category:Wimbledon Championship by year – Wheelchair men's singles": "",
    "Category:Wimbledon Championship by year – Wheelchair quad doubles": "",
    "Category:Wimbledon Championship by year – Wheelchair quad singles": "",
    "Category:Wimbledon Championship by year – Wheelchair women's doubles": "",
    "Category:Wimbledon Championship by year – Wheelchair women's singles": "",
}

test_data = [
    ("test_wheelchair_racers_by_nat", wheelchair_racers_by_nat),
    ("test_wheelchair_3", data2),
]


@pytest.mark.parametrize(
    "category, expected", wheelchair_racers_by_nat.items(), ids=list(wheelchair_racers_by_nat.keys())
)
@pytest.mark.fast
def test_wheelchair_racers_by_nat(category: str, expected: str) -> None:
    label = resolve_arabic_category_label(category)
    assert label == expected


@pytest.mark.parametrize("category, expected", data2.items(), ids=data2.keys())
@pytest.mark.fast
def test_wheelchair_3(category: str, expected: str) -> None:
    label = resolve_arabic_category_label(category)
    assert label == expected


@pytest.mark.dump
@pytest.mark.parametrize("name,data", test_data)
def test_dump_all(name: str, data: str) -> None:
    expected, diff_result = one_dump_test(data, resolve_arabic_category_label)

    dump_diff(diff_result, name)
    assert diff_result == expected, f"Differences found: {len(diff_result):,}, len all :{len(data):,}"



"""Tests for the player and singer job datasets."""

from __future__ import annotations

from ArWikiCats.translations.jobs.jobs_players_list import (
    FOOTBALL_KEYS_PLAYERS,
    PLAYERS_TO_MEN_WOMENS_JOBS,
)


def test_players_dataset_includes_core_sports_roles() -> None:
    """Key sports roles should provide both masculine and feminine labels."""

    football_labels = PLAYERS_TO_MEN_WOMENS_JOBS["footballers"]
    assert football_labels["males"] == "لاعبو كرة قدم"
    assert football_labels["females"] == "لاعبات كرة قدم"


def test_football_key_players_are_registered() -> None:
    """The football helper dictionary should include standard team keys."""

    assert "wide receivers" in FOOTBALL_KEYS_PLAYERS
    assert FOOTBALL_KEYS_PLAYERS["wide receivers"]["males"]



"""
TODO: write tests
"""

import pytest

from ArWikiCats.legacy_bots.ma_bots2.ar_lab_bot import add_in_tab, find_ar_label

fast_data = {
    "00s establishments in the Roman Empire": "تأسيسات عقد 00 في الإمبراطورية الرومانية",
    "1000s disestablishments in Asia": "انحلالات عقد 1000 في آسيا",
    "1990s BC disestablishments in Asia": "انحلالات عقد 1990 ق م في آسيا",
    "1990s disestablishments in Europe": "انحلالات عقد 1990 في أوروبا",
    "April 1983 events in Europe": "أحداث أبريل 1983 في أوروبا",
}


@pytest.mark.parametrize("category, expected", fast_data.items(), ids=fast_data.keys())
@pytest.mark.fast
def test_find_ar_label_fast(category: str, expected: str) -> None:
    label = find_ar_label(category, "in")
    assert label == expected


def test_add_in_tab() -> None:
    # Test with basic inputs
    result = add_in_tab("test label", "test", "from")
    assert isinstance(result, str)

    # Test with different separator value
    result_other = add_in_tab("test label", "test of", "to")
    assert isinstance(result_other, str)

    # Test with empty strings
    result_empty = add_in_tab("", "", "")
    assert isinstance(result_empty, str)


def test_add_in_tab_2() -> None:
    # Test with basic inputs
    result = add_in_tab("test label", "test", "from")
    assert isinstance(result, str)

    # Test with different separator value
    result_other = add_in_tab("test label", "test of", "to")
    assert isinstance(result_other, str)

    # Test with empty strings
    result_empty = add_in_tab("", "", "")
    assert isinstance(result_empty, str)


@pytest.mark.fast
def test_find_ar_label() -> None:
    # Test with basic inputs
    result = find_ar_label("test category", "from")
    assert isinstance(result, str)

    # Test with different parameters
    result_various = find_ar_label("sports category", "in")
    assert isinstance(result_various, str)

    # Test with another valid combination instead of empty strings
    result_safe = find_ar_label("music from france", "from")
    assert isinstance(result_safe, str)



#!/usr/bin/python3
"""
Bot for translating job-related and nationality-based categories.

This module provides functionality for matching and translating categories
related to jobs, nationalities, and multi-sports topics from English to Arabic.

"""

import functools

from ...helps import logger
from ...translations_formats import FormatDataFrom, MultiDataFormatterYearAndFrom
from ..countries_names_resolvers.medalists_resolvers import medalists_data
from ..jobs_resolvers import resolve_jobs_main

medalists_data = dict(
    sorted(
        medalists_data.items(),
        key=lambda k: (-k[0].count(" "), -len(k[0])),
    )
)
formatted_data = {
    "{game} {en_job}": "{ar_job} في {game}",
}


@functools.lru_cache(maxsize=10000)
def match_key_callback(text: str) -> str:
    orgtext = text
    text = text.replace("{game} ", "")
    logger.debug(f" match_key_callback: processed {orgtext=} into {text=}")
    return text.strip()


def get_game_label(text: str) -> str:
    label = medalists_data.get(text.lower(), "")
    logger.debug(f" get_game_label: for {text=} found {label=}")
    return label


def match_game_key(category_lower: str) -> str:
    logger.debug(f" match_game_key: category_lower: {category_lower}")
    for sport_prefix, sport_label in medalists_data.items():
        prefix_pattern = f"{sport_prefix} ".lower()
        if category_lower.startswith(prefix_pattern):
            logger.debug(
                f'jobs_in_multi_sports match: prefix="{prefix_pattern}", '
                f'label="{sport_label}", sport_prefix="{sport_prefix}"'
            )
            return sport_prefix
    return ""


def resolve_jobs_main_2(category: str) -> str:
    """A wrapper around resolve_jobs_main to handle empty results."""
    logger.debug(f"resolve_jobs_main_2: category: {category}")
    return ""


@functools.lru_cache(maxsize=1)
def multi_bot_v4() -> MultiDataFormatterYearAndFrom:
    country_bot = FormatDataFrom(
        formatted_data=formatted_data,
        key_placeholder="{en_job}",
        value_placeholder="{ar_job}",
        search_callback=resolve_jobs_main,
        # search_callback=resolve_jobs_main_2,
        match_key_callback=match_key_callback,
    )
    game_bot = FormatDataFrom(
        formatted_data={},
        key_placeholder="{game}",
        value_placeholder="{game}",
        search_callback=get_game_label,
        match_key_callback=match_game_key,
    )
    return MultiDataFormatterYearAndFrom(
        country_bot=country_bot,
        year_bot=game_bot,
        other_key_first=True,
    )


@functools.lru_cache(maxsize=10000)
def jobs_in_multi_sports(category: str) -> str:
    category = category.lower().replace("category:", "")
    category = category.replace("_", " ").lower()
    category = category.replace("olympics", "olympic")

    logger.debug(f"<<yellow>> start jobs_in_multi_sports: {category=}")
    yc_bot = multi_bot_v4()

    result = yc_bot.search_all_category(category)

    logger.info_if_or_debug(f"<<yellow>> end jobs_in_multi_sports: {category=}, {result=}", result)
    return result or ""


__all__ = [
    "jobs_in_multi_sports",
]



import pytest

from ArWikiCats.format_bots.pf_keys import change_key_mappings_replacements, change_key_secondary_replacements


class TestChangeKeyMappingsReplacements:
    test_data = {
        "category:american television series based on non-american television series": "category:american television series basedon non-american television series",
        "category:australian television series based on non-australian television series": "category:australian television series basedon non-australian television series",
        "category:austrian television series based on non-austrian television series": "category:austrian television series basedon non-austrian television series",
        "category:belgian television series based on non-belgian television series": "category:belgian television series basedon non-belgian television series",
        "category:brazilian television series based on non-brazilian television series": "category:brazilian television series basedon non-brazilian television series",
        "category:british television series based on non-british television series": "category:british television series basedon non-british television series",
        "category:bulgarian television series based on non-bulgarian television series": "category:bulgarian television series basedon non-bulgarian television series",
        "comedy- test": "comedy  test",
        "something comedy- something": "something comedy  something",
        # Test cases for each key in CHANGE_KEY_MAPPINGS
        " labor test": " labour test",
        "labor test": "labour test",
        "something labor something": "something labour something",
        "labor": "labour",
        " labor ": " labour ",
        "war of test": "war-of test",
        "something war of something": "something war-of something",
        "war of yemen": "war-of yemen",
        "category:players in test": "category:playerss in test",
        "players in test": "playerss in test",
        "something players in something": "something playerss in something",
        "something players in": "something playerss in",
        "players in": "playerss in",
        "tennis test": "tennis test",
        "something tennis something": "something tennis something",
        "tennis": "tennis",
        "accidents and incidents test": "accidents-and-incidents test",
        "something accidents and incidents something": "something accidents-and-incidents something",
        "accidents and incidents": "accidents-and-incidents",
        "adaptations of works test": "adaptations-of-works test",
        "something adaptations of works something": "something adaptations-of-works something",
        "adaptations of works": "adaptations-of-works",
        "africa cup of nations test": "africa cup-of-nations test",
        "something africa cup of nations something": "something africa cup-of-nations something",
        "africa cup of nations": "africa cup-of-nations",
        "african american test": "africanamerican test",
        "something african american something": "something africanamerican something",
        "african american": "africanamerican",
        "african-american test": "africanamerican test",
        "something african-american something": "something africanamerican something",
        "african-american": "africanamerican",
        "ancient greek test": "ancient-greek test",
        "something ancient greek something": "something ancient-greek something",
        "ancient greek": "ancient-greek",
        "ancient macedonian test": "ancient-macedonian test",
        "something ancient macedonian something": "something ancient-macedonian something",
        "ancient macedonian": "ancient-macedonian",
        "ancient roman test": "ancient-roman test",
        "something ancient roman something": "something ancient-roman something",
        "ancient roman": "ancient-roman",
        "architecture schools test": "architecture-schools test",
        "something architecture schools something": "something architecture-schools something",
        "architecture schools": "architecture-schools",
        "association football afc test": "association-football afc test",
        "something association football afc something": "something association-football afc something",
        "association football afc": "association-football afc",
        "athletes (track and field) test": "track and field athletes test",
        "something athletes (track and field) something": "something track and field athletes something",
        "athletes (track and field)": "track and field athletes",
        "bodies of water test": "bodies-of-water test",
        "something bodies of water something": "something bodies-of-water something",
        "bodies of water": "bodies-of-water",
        "british hong kong test": "british-hong-kong test",
        "something british hong kong something": "something british-hong-kong something",
        "british hong kong": "british-hong-kong",
        "built by test": "built-by test",
        "something built by something": "something built-by something",
        "built by": "built-by",
        "built in test": "built-in test",
        "something built in something": "something built-in something",
        "built in": "built-in",
        "by car bomb test": "by-car-bomb test",
        "something by car bomb something": "something by-car-bomb something",
        "by car bomb": "by-car-bomb",
        "by firearm in test": "by-firearm-in test",
        "something by firearm in something": "something by-firearm-in something",
        "by firearm in": "by-firearm-in",
        "canadian football test": "canadian-football test",
        "something canadian football something": "something canadian-football something",
        "canadian football": "canadian-football",
        "canton of test": "canton-of test",
        "something canton of something": "something canton-of something",
        "canton of": "canton-of",
        "caribbean people test": "caribbeans people test",
        "something caribbean people something": "something caribbeans people something",
        "caribbean people": "caribbeans people",
        "child soldiers test": "child-soldiers test",
        "something child soldiers something": "something child-soldiers something",
        "child soldiers": "child-soldiers",
        "city of liverpool f.c. test": "city-of-liverpool f.c. test",
        "something city of liverpool f.c. something": "something city-of-liverpool f.c. something",
        "city of liverpool f.c.": "city-of-liverpool f.c.",
        "city of london test": "city-of-london test",
        "something city of london something": "something city-of-london something",
        "city of london": "city-of-london",
        "colony of test": "colony-of test",
        "something colony of something": "something colony-of something",
        "colony of": "colony-of",
        "comedy-": "comedy",
        "county of test": "county-of test",
        "something county of something": "something county-of something",
        "county of": "county-of",
        "crown of test": "crown-of test",
        "something crown of something": "something crown-of something",
        "crown of": "crown-of",
        "domain of test": "domain-of test",
        "something domain of something": "something domain-of something",
        "domain of": "domain-of",
        "duchy of test": "duchy-of test",
        "something duchy of something": "something duchy-of something",
        "duchy of": "duchy-of",
        "early modern test": "early-modern test",
        "something early modern something": "something early-modern something",
        "early modern": "early-modern",
        "emirate of test": "emirate-of test",
        "something emirate of something": "something emirate-of something",
        "emirate of": "emirate-of",
        "executions by test": "executions in test",
        "something executions by something": "something executions in something",
        "executions by": "executions in",
        "football (soccer) test": "football test",
        "something football (soccer) something": "something football something",
        "football (soccer)": "football",
        "for member of parliament test": "for member-of-parliament test",
        "something for member of parliament something": "something for member-of-parliament something",
        "for member of parliament": "for member-of-parliament",
        "future elections test": "future-elections test",
        "something future elections something": "something future-elections something",
        "future elections": "future-elections",
        "general elections test": "general-elections test",
        "something general elections something": "something general-elections something",
        "general elections": "general-elections",
        "elections, ": "elections ",
        "elections,  test": "elections  test",
        "something elections,  something": "something elections  something",
        "governance of policing test": "governance policing test",
        "something governance of policing something": "something governance policing something",
        "governance of policing": "governance policing",
        "harrow on hill test": "harrow-on-hill test",
        "something harrow on hill something": "something harrow-on-hill something",
        "harrow on hill": "harrow-on-hill",
        "health care test": "healthcare test",
        "something health care something": "something healthcare something",
        "health care": "healthcare",
        "imprisoned in test": "imprisoned-in test",
        "something imprisoned in something": "something imprisoned-in something",
        "imprisoned in": "imprisoned-in",
        "in northern ireland test": "in northern-ireland test",
        "something in northern ireland something": "something in northern-ireland something",
        "in northern ireland": "in northern-ireland",
        "in sport in test": "in-sport-in test",
        "something in sport in something": "something in-sport-in something",
        "in sport in": "in-sport-in",
        "in sports in test": "in-sports-in test",
        "something in sports in something": "something in-sports-in something",
        "in sports in": "in-sports-in",
        "isle of test": "isle-of test",
        "something isle of something": "something isle-of something",
        "isle of": "isle-of",
        "killed in action test": "killed-in-action test",
        "something killed in action something": "something killed-in-action something",
        "killed in action": "killed-in-action",
        "kingdom of test": "kingdom-of test",
        "something kingdom of something": "something kingdom-of something",
        "kingdom of": "kingdom-of",
        "labour and social security test": "labour-and-social security test",
        "something labour and social security something": "something labour-and-social security something",
        "labour and social security": "labour-and-social security",
        "launched by test": "launched-by test",
        "something launched by something": "something launched-by something",
        "launched by": "launched-by",
        "launched in test": "launched-in test",
        "something launched in something": "something launched-in something",
        "launched in": "launched-in",
        "local elections test": "local-elections test",
        "something local elections something": "something local-elections something",
        "local elections": "local-elections",
        "manufactured by test": "manufactured-by test",
        "something manufactured by something": "something manufactured-by something",
        "manufactured by": "manufactured-by",
        "manufactured in test": "manufactured-in test",
        "something manufactured in something": "something manufactured-in something",
        "manufactured in": "manufactured-in",
        "march of test": "march-of test",
        "something march of something": "something march-of something",
        "march of": "march-of",
        "margraviate of test": "margraviate-of test",
        "something margraviate of something": "something margraviate-of something",
        "margraviate of": "margraviate-of",
        "medallists test": "medalists test",
        "something medallists something": "something medalists something",
        "medallists": "medalists",
        "military equipment test": "military-equipment test",
        "something military equipment something": "something military-equipment something",
        "military equipment": "military-equipment",
        "military terminology test": "military-terminology test",
        "something military terminology something": "something military-terminology something",
        "military terminology": "military-terminology",
        "ministers for test": "ministers-for test",
        "something ministers for something": "something ministers-for something",
        "ministers for": "ministers-for",
        "murderers of children test": "murderersofchildren test",
        "something murderers of children something": "something murderersofchildren something",
        "murderers of children": "murderersofchildren",
        "national register of historic places test": "national-register-of-historic-places test",
        "something national register of historic places something": "something national-register-of-historic-places something",
        "national register of historic places": "national-register-of-historic-places",
        "presidential elections test": "presidential-elections test",
        "something presidential elections something": "something presidential-elections something",
        "presidential elections": "presidential-elections",
        "presidential primaries test": "presidential-primaries test",
        "something presidential primaries something": "something presidential-primaries something",
        "presidential primaries": "presidential-primaries",
        "prisoners of conscience test": "prisoners-of-conscience test",
        "something prisoners of conscience something": "something prisoners-of-conscience something",
        "prisoners of conscience": "prisoners-of-conscience",
        "protectorate of test": "protectorate-of test",
        "something protectorate of something": "something protectorate-of something",
        "protectorate of": "protectorate-of",
        "publishers (people) test": "publisherspeople test",
        "something publishers (people) something": "something publisherspeople something",
        "publishers (people)": "publisherspeople",
        "realm of test": "realm-of test",
        "something realm of something": "something realm-of something",
        "realm of": "realm-of",
        "refusing to convert to christianity test": "refusing-to-convert-to-christianity test",
        "something refusing to convert to christianity something": "something refusing-to-convert-to-christianity something",
        "refusing to convert to christianity": "refusing-to-convert-to-christianity",
        "refusing to convert to islam test": "refusing-to-convert-to-islam test",
        "something refusing to convert to islam something": "something refusing-to-convert-to-islam something",
        "refusing to convert to islam": "refusing-to-convert-to-islam",
        "republic of test": "republic-of test",
        "something republic of something": "something republic-of something",
        "republic of": "republic-of",
        "saudi arabian test": "saudiarabian test",
        "something saudi arabian something": "something saudiarabian something",
        "saudi arabian": "saudiarabian",
        "scholars of islam test": "scholars-of-islam test",
        "something scholars of islam something": "something scholars-of-islam something",
        "scholars of islam": "scholars-of-islam",
        "shot dead by law enforcement officers test": "shot dead-by-law enforcement officers test",
        "something shot dead by law enforcement officers something": "something shot dead-by-law enforcement officers something",
        "shot dead by law enforcement officers": "shot dead-by-law enforcement officers",
        "sport ministers test": "sport-ministers test",
        "something sport ministers something": "something sport-ministers something",
        "sport ministers": "sport-ministers",
        "sports culture test": "sports-culture test",
        "something sports culture something": "something sports-culture something",
        "sports culture": "sports-culture",
        "sports media test": "sports-media test",
        "something sports media something": "something sports-media something",
        "sports media": "sports-media",
        "sports ministers test": "sports-ministers test",
        "something sports ministers something": "something sports-ministers something",
        "sports ministers": "sports-ministers",
        "sportspeople test": "sports-people test",
        "something sportspeople something": "something sports-people something",
        "sportspeople": "sports-people",
        "states of test": "states-of test",
        "something states of something": "something states-of something",
        "states of": "states-of",
        "television films debuts test": "television films-debuts test",
        "something television films debuts something": "something television films-debuts something",
        "television films debuts": "television films-debuts",
        "television films endings test": "television films-endings test",
        "something television films endings something": "something television films-endings something",
        "television films endings": "television films-endings",
        "television miniseries debuts test": "television miniseries-debuts test",
        "something television miniseries debuts something": "something television miniseries-debuts something",
        "television miniseries debuts": "television miniseries-debuts",
        "television miniseries endings test": "television miniseries-endings test",
        "something television miniseries endings something": "something television miniseries-endings something",
        "television miniseries endings": "television miniseries-endings",
        "television seasons test": "television-seasons test",
        "something television seasons something": "something television-seasons something",
        "television seasons": "television-seasons",
        "television series debuts test": "television series-debuts test",
        "something television series debuts something": "something television series-debuts something",
        "television series debuts": "television series-debuts",
        "television series endings test": "television series-endings test",
        "something television series endings something": "something television series-endings something",
        "television series endings": "television series-endings",
        "transferred from test": "transferred-from test",
        "something transferred from something": "something transferred-from something",
        "transferred from": "transferred-from",
        "united states house of representatives test": "united states house-of-representatives test",
        "something united states house of representatives something": "something united states house-of-representatives something",
        "united states house of representatives": "united states house-of-representatives",
        "university of reading test": "university-of-reading test",
        "something university of reading something": "something university-of-reading something",
        "university of reading": "university-of-reading",
        "university of science and technology test": "university-of-science and technology test",
        "something university of science and technology something": "something university-of-science and technology something",
        "university of science and technology": "university-of-science and technology",
        "university of technology test": "university-of-technology test",
        "something university of technology something": "something university-of-technology something",
        "university of technology": "university-of-technology",
        "us open tennis test": "us open tennis test",
        "something us open tennis something": "something us open tennis something",
        "us open tennis": "us open tennis",
        "viceroyalty of test": "viceroyalty-of test",
        "something viceroyalty of something": "something viceroyalty-of something",
        "viceroyalty of": "viceroyalty-of",
        "west coast of united states test": "west coast-of-united states test",
        "something west coast of united states something": "something west coast-of-united states something",
        "west coast of united states": "west coast-of-united states",
        "world championships in athletics test": "world championships-in-athletics test",
        "something world championships in athletics something": "something world championships-in-athletics something",
        "world championships in athletics": "world championships-in-athletics",
        "world war i test": "world-war-i test",
        "something world war i something": "something world-war-i something",
        "world war i": "world-war-i",
        "world war ii test": "world-war-ii test",
        "something world war ii something": "something world-war-ii something",
        "world war ii": "world-war-ii",
        "’ test": "' test",
        "something ’ something": "something ' something",
        "’": "'",
        "defunct privately held companies test": "defunct-privately held-companies test",
        "something defunct privately held companies something": "something defunct-privately held-companies something",
        "defunct privately held companies": "defunct-privately held-companies",
        " - men's tournament test": " mens tournament test",
        " - men's tournament": " mens tournament",
        " - women's tournament test": " womens tournament test",
        " - women's tournament": " womens tournament",
        " – men's tournament test": " mens tournament test",
        " – men's tournament": " mens tournament",
        " – women's tournament test": " womens tournament test",
        " – women's tournament": " womens tournament",
        "category: - men's tournament test": "category: mens tournament test",
        "category: - women's tournament test": "category: womens tournament test",
        "something - men's tournament something": "something mens tournament something",
        "something - men's tournament": "something mens tournament",
        "something - women's tournament something": "something womens tournament something",
        "something - women's tournament": "something womens tournament",
        "something – men's tournament something": "something mens tournament something",
        "something – women's tournament something": "something womens tournament something",
    }

    @pytest.mark.parametrize("input_text,expected", test_data.items(), ids=test_data.keys())
    def test_change_key_mappings_replacements(self, input_text: str, expected: str) -> None:
        result = change_key_mappings_replacements(input_text)
        assert result.strip() == expected.strip()


class TestChangeKeySecondaryReplacements:
    tests_data_secondary = {
        "category:schools for deaf in new york (state)": "category:schools for-deaf in new york (state)",
        "category:royal national college for blind": "category:royal national college for-blind",
        "category:royal national college for blind by city": "category:royal national college for-blind by city",
        "category:marathi films remade in other languages": "category:marathi films remade-in other languages",
    }

    @pytest.mark.parametrize("input_text,expected", tests_data_secondary.items(), ids=tests_data_secondary.keys())
    def test_change_key_secondary_mappings_replacements(self, input_text: str, expected: str) -> None:
        result = change_key_secondary_replacements(input_text)
        assert result.strip() == expected.strip()

    mens_events = {
        "Category:2010s events at Games": "Category:2010s events in Games",
        "Category:men’s events at 2010 Commonwealth Games": "Category:mens-events in 2010 Commonwealth Games",
        "Category:men's events at Commonwealth Games": "Category:mens-events in Commonwealth Games",
        "Category:2020 men's events Games": "Category:2020 mens-events Games",
    }

    @pytest.mark.parametrize("input_text,expected", mens_events.items(), ids=mens_events.keys())
    def test_mens_events(self, input_text: str, expected: str) -> None:
        result = change_key_secondary_replacements(input_text)
        assert result.strip() == expected.strip()

    womens_footballers = {
        "Category:Brazilian expatriate women's footballers": "Category:Brazilian expatriate female footballers",
        "Category:Women's footballers in Kazakhstan by club": "Category:Women's footballers in Kazakhstan by club",
        "Category:2020 Women's footballers by club": "Category:2020 Women's footballers by club",
    }

    @pytest.mark.parametrize("input_text,expected", womens_footballers.items(), ids=womens_footballers.keys())
    def test_womens_footballers(self, input_text: str, expected: str) -> None:
        result = change_key_secondary_replacements(input_text)
        assert result.strip() == expected.strip()



#!/usr/bin/python3
"""Integration tests for v3i translations resolvers validating country, year, and combined formatters."""

import pytest
from load_one_data import dump_diff, dump_diff_text, one_dump_test

from ArWikiCats.new_resolvers.translations_resolvers_v3i.resolve_v3i import get_job_label

test_0 = {}

test_data_standard = {
    "sports-people": "رياضيون",
    "sportspeople": "رياضيون",
    "botanists": "علماء نباتات",
    "actors": "ممثلون",
    "actresses": "ممثلات",
    "architects": "معماريون",
    "artisans": "حرفيون",
    "artists": "فنانون",
    "astronomers": "فلكيون",
    "autobiographers": "كتاب سيرة ذاتية",
    "ballet dancers": "راقصو باليه",
    "bass guitarists": "عازفو غيتار باس",
    "biographers": "كتاب سيرة",
    "biologists": "علماء أحياء",
    "businesspeople": "شخصيات أعمال",
    "businesswomen": "سيدات أعمال",
    "chemists": "كيميائيون",
    "chess players": "لاعبو شطرنج",
    "civil servants": "موظفو خدمة مدنية",
    "classical composers": "ملحنون كلاسيكيون",
    "classical musicians": "موسيقيون كلاسيكيون",
    "classical pianists": "عازفو بيانو كلاسيكيون",
    "clergy": "رجال دين",
    "comedians": "كوميديون",
    "composers": "ملحنون",
    "criminals": "مجرمون",
    "dancers": "راقصون",
    "deaths": "وفيات",
    "diarists": "كتاب يوميات",
    "diplomats": "دبلوماسيون",
    "dramatists and playwrights": "كتاب دراما ومسرح",
    "drummers": "طبالون",
    "educators": "معلمون",
    "engineers": "مهندسون",
    "engravers": "نقاشون",
    "explorers": "مستكشفون",
    "folk musicians": "موسيقيو فولك",
    "guitarists": "عازفو قيثارة",
    "historians": "مؤرخون",
    "illustrators": "رسامون توضيحيون",
    "Jews": "يهود",
    "journalists": "صحفيون",
    "jurists": "حقوقيون",
    "landowners": "حائزو أراضي",
    "lawyers": "محامون",
    "LGBTQ people": "أعلام إل جي بي تي كيو",
    "male actors": "ممثلون ذكور",
    "male artists": "فنانون ذكور",
    "male composers": "ملحنون ذكور",
    "male musicians": "موسيقيون ذكور",
    "male singers": "مغنون ذكور",
    "male writers": "كتاب ذكور",
    "mathematicians": "رياضياتيون",
    "medical doctors": "أطباء",
    "memoirists": "كتاب مذكرات",
    "military personnel": "أفراد عسكريون",
    "musicians": "موسيقيون",
    "naturalists": "علماء طبيعة",
    "nobility": "نبلاء",
    "non-fiction writers": "كتاب غير روائيين",
    "novelists": "روائيون",
    "opera singers": "مغنو أوبرا",
    "painters": "رسامون",
    "philosophers": "فلاسفة",
    "photographers": "مصورون",
    "physicians": "أطباء",
    "physicists": "فيزيائيون",
    "pianists": "عازفو بيانو",
    "poets": "شعراء",
    "politicians": "سياسيون",
    "princes": "أمراء",
    "publishers (people)": "ناشرون",
    "rabbis": "حاخامات",
    "scholars": "دارسون",
    "scientists": "علماء",
    "screenwriters": "كتاب سيناريو",
    "sculptors": "نحاتون",
    "short story writers": "كتاب قصة قصيرة",
    "singer-songwriters": "مغنون وكتاب أغاني",
    "singers": "مغنون",
    "songwriters": "كتاب أغان",
    "sportsmen": "رياضيون رجال",
    "sportswomen": "رياضيات",
    "translators": "مترجمون",
    "violinists": "عازفو كمان",
    "women artists": "فنانات",
    "women composers": "ملحنات",
    "women educators": "معلمات",
    "women journalists": "صحفيات",
    "women lawyers": "محاميات",
    "women medical doctors": "طبيبات",
    "women musicians": "موسيقيات",
    "women opera singers": "مغنيات أوبرا",
    "women painters": "رسامات",
    "women politicians": "سياسيات",
    "women scientists": "عالمات",
    "women singers": "مغنيات",
    "women writers": "كاتبات",
    "women": "نساء",
    "writers": "كتاب",
    "zoologists": "علماء حيوانات",
}


@pytest.mark.parametrize("category,expected", test_data_standard.items(), ids=test_data_standard.keys())
def test_get_job_label_1(category: str, expected: str) -> None:
    """
    Test
    """
    result = get_job_label(category)
    assert result == expected


to_test = [
    ("test_get_job_label_1", test_data_standard),
]


@pytest.mark.parametrize("name,data", to_test)
@pytest.mark.dump
def test_dump_all(name: str, data: dict[str, str]) -> None:
    expected, diff_result = one_dump_test(data, get_job_label)
    dump_diff(diff_result, name)
    # dump_diff_text(expected, diff_result, name)
    assert diff_result == expected, f"Differences found: {len(diff_result):,}, len all :{len(data):,}"



#!/usr/bin/python3
"""
"""

add_in_to_country = [
    "solar eclipses",
]
# ---
Keep_it_last = [
    "remakes of",
]
# ---
Keep_it_frist = [
    "spaceflight",
    "lists of",
    # "people associated with" ,
    "actors in",
    "male actors in",
    "centuries in",
    "centuries",
    "qualification for",
    "participants",
    "seasons in",
    "seasons",
    "works by",
    "deaths by",
]

Add_in_table = [
    "historical documents",
    "road incidents",
    "racehorse deaths",
    "animal deaths",
    "documents",
    "sports awards",
    "military alliances",
    "illuminated manuscripts",
    "biblical manuscripts",
    "censuses",  # تعداد السكان
]

__all__ = [
    "add_in_to_country",
    "Keep_it_last",
    "Keep_it_frist",
    "Add_in_table",
]



#!/usr/bin/python3
"""
TODO: merge with sports_resolvers/raw_sports.py
"""

import functools

from ...helps import dump_data, logger
from ...new.handle_suffixes import resolve_sport_category_suffix_with_mapping, resolve_suffix_with_mapping_genders
from ...translations.sports.Sport_key import SPORT_KEY_RECORDS
from ...translations_formats import FormatDataV2

teams_2025_sample = {
    "{sport} people": "أعلام {sport_jobs}",
    "{sport} squads": "تشكيلات {sport_jobs}",
    "{sport} finals": "نهائيات {sport_jobs}",
    "{sport} positions": "مراكز {sport_jobs}",
    "{sport} tournaments": "بطولات {sport_jobs}",
    "{sport} films": "أفلام {sport_jobs}",
    "{sport} teams": "فرق {sport_jobs}",
    "{sport} venues": "ملاعب {sport_jobs}",
    "{sport} clubs": "أندية {sport_jobs}",
    "{sport} organizations": "منظمات {sport_jobs}",
}

mappings_data: dict[str, str] = {
    "squads": "تشكيلات",
    "finals": "نهائيات",
    "positions": "مراكز",
    "tournaments": "بطولات",
    "films": "أفلام",
    "teams": "فرق",
    "venues": "ملاعب",
    "clubs": "أندية",
    "clubs and teams": "أندية وفرق",
    "organizations": "منظمات",
    "non-profit organizations": "منظمات غير ربحية",
    "non-profit publishers": "ناشرون غير ربحيون",
    "organisations": "منظمات",
    "events": "أحداث",
    "scouts": "كشافة",
    "leagues": "دوريات",
    "results": "نتائج",
    "matches": "مباريات",
    "navigational boxes": "صناديق تصفح",
    "lists": "قوائم",
    "home stadiums": "ملاعب",
    "templates": "قوالب",
    "rivalries": "دربيات",
    "champions": "أبطال",
    "competitions": "منافسات",
    "statistics": "إحصائيات",
    "records": "سجلات",
    "records and statistics": "سجلات وإحصائيات",
    "manager history": "تاريخ مدربو",
    "trainers": "مدربو",
    "coaches": "مدربو",
    "managers": "مدربو",
    "people": "أعلام",
    "umpires": "حكام",
    "referees": "حكام",
    "directors": "مدراء",
}

teams_2025 = {
    "{sport}": "{sport_jobs}",
    # "{sport}": "{sport_label}",
    "amateur {sport}": "{sport_jobs} للهواة",
    "mens youth {sport}": "{sport_jobs} للشباب",
    "mens {sport}": "{sport_jobs} رجالية",
    "womens youth {sport}": "{sport_jobs} للشابات",
    "womens {sport}": "{sport_jobs} نسائية",
    "youth {sport}": "{sport_jobs} شبابية",
}

FOOTBALL_KEYS_PLAYERS = {
    "journalists": {"males": "صحفيو", "females": "صحفيات"},
    "players": {"males": "لاعبو", "females": "لاعبات"},
    "placekickers": {"males": "مسددو", "females": "مسددات"},
    "kickers": {"males": "راكلو", "females": "راكلات"},
    "defenders": {"males": "مدافعو", "females": "مدافعات"},
    "forwards": {"males": "مهاجمو", "females": "مهاجمات"},
    "fullbacks": {"males": "مدافعو", "females": "مدافعات"},
    "defencemen": {"males": "مدافعو", "females": "مدافعات"},
    "receivers": {"males": "مستقبلو", "females": "مستقبلات"},
    "tackles": {"males": "مصطدمو", "females": "مصطدمات"},
    "sports-people": {"males": "رياضيو", "females": "رياضيات"},
    "utility players": {"males": "لاعبو مراكز متعددة", "females": "لاعبات مراكز متعددة"},
    "wide receivers": {"males": "مستقبلون واسعون", "females": "مستقبلات واسعات"},
    "peoplee": {"males": "أعلام", "females": "أعلام"},
    "scouts": {"males": "كشافة", "females": "كشافة"},
    "halfbacks": {"males": "أظهرة مساعدون", "females": "ظهيرات مساعدات"},
    "quarterbacks": {"males": "أظهرة رباعيون", "females": "ظهيرات رباعيات"},
    "centers": {"males": "لاعبو وسط", "females": "لاعبات وسط"},
    "centres": {"males": "لاعبو وسط", "females": "لاعبات وسط"},
    "midfielders": {"males": "لاعبو وسط", "females": "لاعبات وسط"},
    "drop kickers": {"males": "مسددو ركلات", "females": "مسددات ركلات"},
    "central defenders": {"males": "قلوب دفاع", "females": "مدافعات مركزيات"},
    "inside forwards": {"males": "مهاجمون داخليون", "females": "مهاجمات داخليات"},
    "outside forwards": {"males": "مهاجمون خارجيون", "females": "مهاجمات خارجيات"},
    "small forwards": {"males": "مهاجمون صغيرو الجسم", "females": "مهاجمات صغيرات الجسم"},
    "power forwards": {"males": "مهاجمون أقوياء الجسم", "females": "مهاجمات قويات الجسم"},
    "defensive backs": {"males": "مدافعون خلفيون", "females": "مدافعات خلفيات"},
    "running backs": {"males": "راكضون للخلف", "females": "راكضات للخلف"},
    "linebackers": {"males": "أظهرة", "females": "ظهيرات"},
    "goalkeepers": {"males": "حراس مرمى", "females": "حارسات مرمى"},
    "goaltenders": {"males": "حراس مرمى", "females": "حارسات مرمى"},
    "guards": {"males": "حراس", "females": "حارسات"},
    "shooting guards": {"males": "مدافعون مسددون", "females": "مدافعات مسددات"},
    "point guards": {"males": "لاعبو هجوم خلفي", "females": "لاعبات هجوم خلفي"},
    "offensive linemen": {"males": "مهاجمو خط", "females": "مهاجمات خط"},
    "defensive linemen": {"males": "مدافعو خط", "females": "مدافعات خط"},
    "left wingers": {"males": "أجنحة يسار", "females": "جناحات يسار"},
    "right wingers": {"males": "أجنحة يمين", "females": "جناحات يمين"},
    "wingers": {"males": "أجنحة", "females": "جناحات"},
    "wing halves": {"males": "أنصاف أجنحة", "females": "جناحات نصفيات"},
}

mappings_data = dict(
    sorted(
        mappings_data.items(),
        key=lambda k: (-k[0].count(" "), -len(k[0])),
    )
)

football_keys_players = dict(
    sorted(
        FOOTBALL_KEYS_PLAYERS.items(),
        key=lambda k: (-k[0].count(" "), -len(k[0])),
    )
)

PPP_Keys = {
    "": "",
    "mens": "رجالية",
    "womens": "نسائية",
    "youth": "شبابية",
    "mens youth": "للشباب",
    "womens youth": "للشابات",
    "amateur": "للهواة",
}


@functools.lru_cache(maxsize=1)
def load_v2() -> FormatDataV2:
    """Load and cache the formatter used for 2025 team categories."""

    sports_data = {
        x: {
            "sport_label": v.get("label", ""),
            "sport_team": v.get("team", ""),
            "sport_jobs": v.get("jobs", ""),
        }
        for x, v in SPORT_KEY_RECORDS.items()
        if v.get("jobs")
    }
    sports_data.pop("sports", None)
    bot = FormatDataV2(
        formatted_data=teams_2025,
        data_list=sports_data,
        key_placeholder="{sport}",
    )

    return bot


def fix_result_callable(result: str, category: str, key: str, value: str) -> str:
    if result.startswith("لاعبو ") and "للسيدات" in result:
        result = result.replace("لاعبو ", "لاعبات ")

    if key == "teams" and "national" in category:
        result = result.replace("فرق ", "منتخبات ")

    return result


@functools.lru_cache(maxsize=None)
def _find_teams_2025(category: str, default: str = "") -> str:
    """Search for a 2025 team label, falling back to ``default`` when absent."""
    bot = load_v2()
    return bot.search_all_category(category) or default


@functools.lru_cache(maxsize=10000)
def fix_keys(category: str) -> str:
    category = category.lower().replace("category:", "")
    category = category.replace("'", "")
    category = category.replace("playerss", "players")

    return category.strip()


# @dump_data(1)
def find_teams_2025(category) -> str:
    category = fix_keys(category)

    logger.debug(f"<<yellow>> start find_teams_2025: {category=}")

    if SPORT_KEY_RECORDS.get(category):
        return SPORT_KEY_RECORDS[category].get("label", "")

    result = _find_teams_2025(category)

    if not result:
        result = resolve_sport_category_suffix_with_mapping(
            category=category,
            data=mappings_data,
            callback=_find_teams_2025,
            fix_result_callable=fix_result_callable,
        )

    if not result:
        result = resolve_suffix_with_mapping_genders(
            category=category,
            data=football_keys_players,
            callback=_find_teams_2025,
            fix_result_callable=fix_result_callable,
        )

    logger.info_if_or_debug(f"<<yellow>> end find_teams_2025: {category=}, {result=}", result)
    return result


__all__ = [
    "find_teams_2025",
]
